Package: Landmarking
Title: Analysis using Landmark Models
Version: 1.0.2
Authors@R: c(
    person(given = "Isobel",
           family = "Barrott",
           role = c("aut", "cre"),
           email = "isobel.barrott@gmail.com"),
    person(given = "Jessica",
           family = "Barrett",
           role = c("aut"),
           email = "jessica.barrett@mrc-bsu.cam.ac.uk"),
    person(given = "Ruth",
           family = "Keogh",
           role = c("ctb")),
    person(given = "Michael",
           family = "Sweeting",
           role = c("ctb")),
    person(given = "David",
           family = "Stevens",
           role = c("ctb")))
Description: The landmark approach allows survival predictions to be
	updated dynamically as new measurements from an individual are recorded.
	The idea is to set predefined time points, known as "landmark times",
	and form a model at each landmark time using only the individuals in the
	risk set. This package allows the longitudinal data to be modelled
	either using the last observation carried forward or linear mixed
	effects modelling. There is also the option to model competing risks,
	either through cause-specific Cox regression or Fine-Gray regression.
	To find out more about the methods in this package, please see 
	<https://isobelbarrott.github.io/Landmarking/articles/Landmarking>.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
LazyDataCompression: bzip2
Imports: nlme, riskRegression, dplyr, pec, methods, prodlim, stats,
        survival, mstate, ggplot2,
RoxygenNote: 7.1.2
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, JM
VignetteBuilder: knitr
Depends: R (>= 3.5.0)
URL: https://github.com/isobelbarrott/Landmarking/,
        https://github.com/isobelbarrott/Landmarking
BugReports: https://github.com/isobelbarrott/Landmarking/issues
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-16 18:26:04 UTC; qs251129
Author: Isobel Barrott [aut, cre],
  Jessica Barrett [aut],
  Ruth Keogh [ctb],
  Michael Sweeting [ctb],
  David Stevens [ctb]
Maintainer: Isobel Barrott <isobel.barrott@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-16 21:50:44 UTC
Built: R 4.6.0; ; 2026-05-16 22:46:19 UTC; unix

Package: Landmarking
Title: Analysis using Landmark Models
Version: 1.0.2
Authors@R: c(
    person(given = "Isobel",
           family = "Barrott",
           role = c("aut", "cre"),
           email = "isobel.barrott@gmail.com"),
    person(given = "Jessica",
           family = "Barrett",
           role = c("aut"),
           email = "jessica.barrett@mrc-bsu.cam.ac.uk"),
    person(given = "Ruth",
           family = "Keogh",
           role = c("ctb")),
    person(given = "Michael",
           family = "Sweeting",
           role = c("ctb")),
    person(given = "David",
           family = "Stevens",
           role = c("ctb")))
Description: The landmark approach allows survival predictions to be
	updated dynamically as new measurements from an individual are recorded.
	The idea is to set predefined time points, known as "landmark times",
	and form a model at each landmark time using only the individuals in the
	risk set. This package allows the longitudinal data to be modelled
	either using the last observation carried forward or linear mixed
	effects modelling. There is also the option to model competing risks,
	either through cause-specific Cox regression or Fine-Gray regression.
	To find out more about the methods in this package, please see 
	<https://isobelbarrott.github.io/Landmarking/articles/Landmarking>.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
LazyDataCompression: bzip2
Imports: nlme, riskRegression, dplyr, pec, methods, prodlim, stats,
        survival, mstate, ggplot2,
RoxygenNote: 7.1.2
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, JM
VignetteBuilder: knitr
Depends: R (>= 3.5.0)
URL: https://github.com/isobelbarrott/Landmarking/,
        https://github.com/isobelbarrott/Landmarking
BugReports: https://github.com/isobelbarrott/Landmarking/issues
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-16 18:26:04 UTC; qs251129
Author: Isobel Barrott [aut, cre],
  Jessica Barrett [aut],
  Ruth Keogh [ctb],
  Michael Sweeting [ctb],
  David Stevens [ctb]
Maintainer: Isobel Barrott <isobel.barrott@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-16 21:50:44 UTC
Built: R 4.6.0; ; 2026-05-17 02:18:51 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,landmark)
S3method(predict,landmark)
S3method(print,landmark)
export(add_cv_number)
export(find_LME_risk_set)
export(find_LOCF_risk_set)
export(fit_LME_landmark)
export(fit_LME_longitudinal)
export(fit_LOCF_landmark)
export(fit_LOCF_longitudinal)
export(fit_survival_model)
export(get_model_assessment)
export(mixoutsamp)
export(return_ids_with_LOCF)
importFrom(prodlim,Hist)
importFrom(stats,as.formula)
importFrom(survival,Surv)
importFrom(survival,coxph)

<!-- README.md is generated from README.Rmd. Please edit that file -->

This package is designed to help the user perform dynamic prediction
using a landmark model. Dynamic prediction means that risk predictions
of an event are updated as more longitudinal data is collected. There is
the option to use the last observation carried forward (LOCF) or linear
mixed effects (LME) model in the first stage of the two-stage landmark
model. This package allows the user to account for competing risks by
using either the Fine Gray model or cause-specific model (or the Cox
model) in the second stage of the two-stage landmark model. k-fold
cross-validation can be performed using this package.

For more detailed examples and for an explanation of landmark models,
see <https://isobelbarrott.github.io/Landmarking/>.

## Installation

You can install the development version of this package from
[GitHub](https://github.com/) with:

``` r
install.packages("devtools")
devtools::install_github("isobelbarrott/Landmarking")
```

## A simple example

Below is a simple toy example of how to use this package with the pbc2
dataset from package JM to predict the risk of death for a new patient.

This uses the LOCF method for the first stage of the landmark model and
cause-specific model for the second stage.

``` r
#Load the library and dataset
library(Landmarking)
data(pbc2,package="JM")
#Change levels to make the death the event of interest (event_status=1), transplant the competing risks (event_status=2), and leave censoring (event_status=0)
levels(pbc2$status)<-c("0","2","1")
#Calculate the age of the patient at each assessment (as opposed to time since first assessment)
pbc2$years<-pbc2$years+pbc2$age
#Fit the landmark model
data_model_landmark_LOCF<-fit_LOCF_landmark(data=pbc2,
                                                  x_L=40,
                                                  x_hor=45,
                                                  covariates=c("serBilir","serChol"),
                                                  covariates_time="year",
                                                  individual_id="id",
                                                  event_time="years",
                                                  event_status="status",
                                                  survival_submodel = "cause_specific",
                                                  b=50)
#Define new dataset
newdata<-rbind(data.frame(id=c(313,313,313),year=c(30,32,35),serBilir=c(2.4,2.7,2.6),serChol=c(220,233,234)))
#Return event prediction and LOCF values
predict(object=data_model_landmark_LOCF,x_L=40,x_hor=45,newdata=newdata)
```

<!-- badges: start -->

[![R-CMD-check](https://github.com/isobelbarrott/Landmarking/workflows/R-CMD-check/badge.svg)](https://github.com/isobelbarrott/Landmarking/actions)
[![R-CMD-check](https://github.com/isobelbarrott/Landmarking/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/isobelbarrott/Landmarking/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----figurename, echo=FALSE, fig.cap="*Figure 1: Illustration of a landmark model with six landmark times (ages 45 to 50 years). The blue solid line represents the time interval of longitudinal (repeat measures) data that feeds into the first stage of the two-stage landmark model. The red dashed line represents the time interval of the time-to-event data to be inputted into the survival model which is the second stage of the landmark model. Events after the time horizon need to be censored within this framework.*"----
knitr::include_graphics("../vignettes/landmarking_models.png")

## ----figurename2, echo=FALSE, fig.cap="*Figure 2: Comparison of the LOCF and LME model to predict systolic blood pressure at a landmark age for an individual with 8 repeat measures.*"----
knitr::include_graphics("../vignettes/landmarking_lme_model.png")

## ----figurename3, echo=FALSE, fig.cap=""--------------------------------------
knitr::include_graphics("../vignettes/fit_LOCF_landmark_model.png")

#' Fit a survival sub-model as part of a landmarking analysis
#'
#' This function is a helper function for `fit_LOCF_landmark_model` and `fit_LME_landmark_model`.
#'
#' @param data Data frame containing covariates and time-to-event data, one row for each individual.
#' @template x_hor
#' @template covariates
#' @param cv_name Character string specifying the column name in `data` that indicates cross-validation fold. If no cross-validation is needed, set this parameter to `NA`.
#' @param event_time Character string specifying the column name in `data` which contains the event time
#' @param event_status	Character string specifying the column name in `data` which contains the event status (where 0=censoring, 1=event of interest, if there are competing events these are labelled 2 or above). Events at time x_hor should be labelled censored.
#' @param individual_id Character string specifying the column name in `data` which contains the individual identifiers
#' @template survival_submodel
#' @return List containing `data_survival` and `model_survival`
#'
#' `data_survival` contains the predicted risk of event by the horizon time `x_hor`.
#'
#' `model_survival` contains the outputs from the function used to fit the survival submodel, including the estimated parameters of the model.
#' For a model using cross-validation, `model_survival` contains a list of outputs with each
#' element in the list corresponding to a different cross-validation fold.
#'
#' @details
#'
#' This function fits the survival model from the landmark model framework. The individuals are censored at the time horizon `x_hor` and the survival model is fitted with
#' covariates specified in parameter `covariates`.
#'
#' For the survival model, there are three choices of model:
#' * the standard Cox model, this is a wrapper function for \code{coxph} from the package \code{survival}
#' * the cause-specific model, this is a wrapper function for \code{CSC} from package \code{riskRegression}
#' * the Fine Gray model, this is a wrapper function for \code{FGR} from package \code{riskRegression}
#'
#' The latter two models estimate the probability of the event of interest in the presence of competing events.
#'
#' @author Isobel Barrott \email{isobel.barrott@@gmail.com}
#' @export

fit_survival_model <- function(data,
                               individual_id,
                               cv_name=NA,
                               covariates,
                               event_time,
                               event_status,
                               survival_submodel = c("standard_cox", "cause_specific", "fine_gray"),
                               x_hor) {
  #Checks
  #####
  if (!(inherits(data,"data.frame"))) {
    stop("data should be a dataframe")
  }
  if (!(inherits(x_hor,"numeric"))) {
    stop("x_hor should be numeric")
  }
  for (col in c(covariates,
                event_time,
                event_status,
                individual_id)) {
    if (!(col %in% names(data))) {
      stop(col, " is not a column name in data")
    }
    if(any(is.na(data[[col]]))){
      stop(col, " contains NA values")
    }
  }

  if (!is.na(cv_name)) {
    if (!(cv_name %in% names(data))) {
      stop(cv_name, " is not a column name in data")
    }
    if (any(is.na(data[[cv_name]]))){
      stop("The column ",cv_name, " contains NA values")
    }
  }

  if(is.na(cv_name)){data[["cross_validation_number"]]<-1;cv_name<-"cross_validation_number"}

  survival_submodel <- match.arg(survival_submodel)

  if(!is.null(levels(data[[event_status]]))){
    data[[event_status]]<-as.numeric(levels(data[[event_status]]))[data[[event_status]]]
  }

  if(survival_submodel %in% c("cause_specific", "fine_gray")){
    if(!(setequal(data[[event_status]],0:max(data[[event_status]])))){
      stop("event_status column should contain only values 0, 1, and 2 for cause_specific or fine_gray survival submodel,
        or values 0 and 1 for standard_cox survival submodel")
    }
  }
  if(survival_submodel %in% c("standard_cox")){
    if(!(setequal(data[[event_status]],0:1))){
      stop("event_status column should contain only values 0, 1, and 2 for cause_specific or fine_gray survival submodel,
        or values 0 and 1 for standard_cox survival submodel")
    }
  }

  data[[individual_id]] <- as.factor(data[[individual_id]])

  if(is.na(cv_name)){data[["cross_validation_number"]]<-1;cv_name<-"cross_validation_number"}

    cv_numbers <- unique(data[[cv_name]])
    model <- as.list(cv_numbers)
    names(model) <- cv_numbers
    Surv<-survival::Surv

    #Censor at the time horizon
    data[[event_status]][data[[event_time]] > x_hor] <-
      0
    data[[event_time]][data[[event_time]] > x_hor] <-
      x_hor

    data_cv <- lapply(cv_numbers, function(cv_number) {

      if (length(cv_numbers)==1) {
        data_test <- data
        data_train <- data
      }else{
        data_test <- data[data[[cv_name]] == cv_number, ]
        data_train <- data[data[[cv_name]] != cv_number, ]
      }

      model_survival <- c()

      if (survival_submodel == "standard_cox") {
        formula_survival <-
          as.formula(paste0("Surv(", event_time, ", ", event_status, "==1) ~",
                            paste0(covariates, collapse = "+")))
        model_survival <- coxph(formula_survival, data = data_train,x=TRUE)
        data_test$event_prediction <- as.numeric(riskRegression::predictRisk(model_survival, times = x_hor, newdata = data_test))
      }

      if (survival_submodel == "cause_specific") {
        if(length(unique(data_train[[event_status]]))!=length(unique(data[[event_status]]))){
          stop("Not enough competing risk events to train competing risks model")
        }

        formula_survival <-
          as.formula(paste0("Hist(", event_time, ", ", event_status, ") ~",
                            paste0(covariates, collapse = "+")))
        model_survival <-
          riskRegression::CSC(
            formula_survival,
            data = data_train,
            fitter = "coxph",
            cause = 1
          )
        data_test$event_prediction <- as.numeric(
          riskRegression::predictRisk(
            model_survival,
            cause = 1,
            newdata = data_test,
            times = x_hor
          )
        )
      }

      if (survival_submodel == "fine_gray") {
        if(length(unique(data_train[[event_status]]))!=length(unique(data[[event_status]]))){
          stop("Not enough competing risk events to train competing risks model")
        }
        formula_survival <-
          as.formula(paste0("Hist(", event_time, ", ", event_status, ") ~",
                            paste0(covariates, collapse = "+")))
        model_survival <-
          riskRegression::FGR(formula_survival, data = data_train, cause = 1)
        data_test$event_prediction <-
          as.numeric(
            riskRegression::predictRisk(
              model_survival,
              cause = 1,
              newdata = data_test,
              times = x_hor
            )
          )
      }

      return(list(data_test = data_test, model_survival = model_survival))
    })
    data_survival <-
      data.frame(Reduce(dplyr::bind_rows, lapply(data_cv, `[[`, 1)))
    model_survival <- lapply(data_cv, `[[`, 2)
    names(model_survival)<-cv_numbers
    if (length(cv_numbers)==1) {
      model_survival<-model_survival[[1]]
    }

  data_survival<-data_survival[order(match(data_survival[[individual_id]],data[[individual_id]])),]
  rownames(data_survival)<-NULL
  list(data_survival = data_survival, model_survival = model_survival)
}

#' Predict the risk of an event for a new individual using the landmark model
#'
#' This function predicts the risk of an event for new data using the landmark model fitted by `fit_LME_landmark` or `fit_LOCF_landmark`.
#' The 'event' is defined as event for which `event_status` is 1.
#'
#' @param object Object inheriting the class `landmark`, this should be the output from either `fit_LME_landmark` or `fit_LOCF_landmark`. It should contain a list
#' of landmark models corresponding to different landmark times `x_L`.
#' @param x_L 	Numeric specifying the landmark time. This indicates which landmark model in `object` to use.
#' @param x_hor Numeric specifying the horizon time. The function assesses the risk of event before this time.
#' @param newdata Data frame containing new data to return the risk prediction of the event of interest. The data should be in in long format
#' and the columns must contain the covariates and time variables that are used to fit the model.
#' For the LME model this the variables `predictors_LME`, `responses_LME`, `predictors_LME_time`, and
#' `responses_LME_time`. For the LOCF model this is `covariates` and `covariates_time`.
#' @param cv_fold If cross validation is used to fit `fit_LME_landmark` or `fit_LOCF_landmark`, then the cross validation fold to use when making risk predictions needs to be specified.
#' @param \dots Arguments passed on to `riskRegression::predictRisk`
#' @return Data frame `newdata` updated to contained a new column `event_prediction`
#' @examples
#' library(Landmarking)
#' data(data_repeat_outcomes)
#' data_model_landmark_LOCF <-
#'   fit_LOCF_landmark(
#'     data_long = data_repeat_outcomes,
#'     x_L = c(60, 61),
#'     x_hor = c(65, 66),
#'     covariates =
#'       c("ethnicity", "smoking", "diabetes", "sbp_stnd", "tchdl_stnd"),
#'     covariates_time =
#'       c(rep("response_time_sbp_stnd", 4), "response_time_tchdl_stnd"),
#'     k = 10,
#'     individual_id = "id",
#'     event_time = "event_time",
#'     event_status = "event_status",
#'     survival_submodel = "cause_specific"
#'   )
#'  newdata <-
#'    rbind(
#'      data.frame(
#'        id = c(3001, 3001, 3001),
#'        response_time_sbp_stnd = c(57, 58, 59),
#'        smoking = c(0, 0, 0),
#'        diabetes = c(0, 0, 0),
#'        ethnicity = c("Indian", "Indian", "Indian"),
#'        sbp_stnd = c(0.45, 0.87, 0.85),
#'        tchdl_stnd = c(-0.7, 0.24, 0.3),
#'        response_time_tchdl_stnd = c(57, 58, 59)
#'      )
#'    )
#'  predict(object=data_model_landmark_LOCF,x_L=60,x_hor=62,newdata=newdata,cv_fold=1)
#' @export
predict.landmark <- function(object, x_L, x_hor, newdata, cv_fold = NA, ...) {
  if (!inherits(object,"landmark")) {
    stop("object must have class 'landmark'")
  }
  if (!(as.character(x_L) %in% names(object))) {
    stop("x_L must be the name of an element in 'object'")
  }

  if (!(inherits(x_hor,"numeric"))) {
    stop("x_hor should have class numeric")
  }

  model_longitudinal <- object[[as.character(x_L)]]$model_longitudinal

  if (model_longitudinal == "LOCF") {
    call <- as.list(object[[as.character(x_L)]]$call)

    covariates <- eval(call$covariates)
    covariates_time <- eval(call$covariates_time)
    individual_id <- eval(call$individual_id)
    for (covariate in covariates) {
      if (is.factor(object[[as.character(x_L)]]$data[[covariate]])) {
        newdata[[covariate]] <-
          factor(newdata[[covariate]], levels = levels(object[[as.character(x_L)]]$data[[covariate]]))
      }
    }

    data_model_longitudinal <-
      fit_LOCF_longitudinal(
        data_long = newdata,
        x_L = x_L,
        covariates = covariates,
        covariates_time =
          covariates_time,
        individual_id =
          individual_id
      )
    data_longitudinal <- data_model_longitudinal$data_longitudinal
  }

  if (model_longitudinal == "LME") {
    call <- as.list(object[[as.character(x_L)]]$call)

    responses_LME <- eval(call$responses_LME)
    predictors_LME <- eval(call$predictors_LME)
    predictors_LME_time <- eval(call$predictors_LME_time)
    responses_LME_time <- eval(call$responses_LME_time)
    individual_id <- eval(call$individual_id)
    random_slope_survival <-
      ifelse(
        is.null(eval(call$random_slope_survival)),
        TRUE,
        eval(call$random_slope_survival)
      )

    for (predictor_LME in predictors_LME) {
      if (is.factor(object[[as.character(x_L)]]$data[[predictor_LME]])) {
        newdata[[predictor_LME]] <-
          factor(newdata[[predictor_LME]], levels = levels(object[[as.character(x_L)]]$data[[predictor_LME]]))
      }
    }

    if (!is.na(cv_fold)) {
      model_LME <-
        object[[as.character(x_L)]]$model_LME[[as.character(cv_fold)]]
      model_LME_standardise_time <-
        object[[as.character(x_L)]]$model_LME_standardise_time[[as.character(cv_fold)]]
    }else{
      model_LME <- object[[as.character(x_L)]]$model_LME
      model_LME_standardise_time <-
        object[[as.character(x_L)]]$model_LME_standardise_time
    }

    data_model_longitudinal <-
      fit_LOCF_longitudinal(
        data_long = newdata,
        x_L = x_L,
        covariates = predictors_LME,
        covariates_time =
          predictors_LME_time,
        individual_id =
          individual_id
      )
    data_LOCF <- data_model_longitudinal$data_longitudinal

    response_type <-
      Reduce(c, lapply(responses_LME, function(i) {
        rep(i, dim(data_LOCF)[1])
      }))
    response <- as.numeric(rep(NA, length(response_type)))
    response_time <- as.numeric(rep(x_L, length(response_type)))
    data_predictors_LME <-
      do.call("rbind", replicate(
        n = length(responses_LME),
        data_LOCF[, c(individual_id, predictors_LME)],
        simplify = FALSE
      ))
    data_LOCF <-
      data.frame(data_predictors_LME,
                 response_type,
                 response,
                 response_time,
                 predict = 1)

    #Create validation and development dataset
    #####
    response_type <-
      Reduce(c, lapply(responses_LME, function(i) {
        rep(i, dim(newdata)[1])
      }))
    response <-
      as.numeric(Reduce(c, lapply(1:length(responses_LME), function(i) {
        newdata[, responses_LME[i]]
      })))
    response_time <-
      as.numeric(Reduce(c, lapply(1:length(responses_LME), function(i) {
        newdata[, responses_LME_time[i]]
      })))
    data_predictors_LME <-
      do.call("rbind", replicate(
        n = length(responses_LME),
        newdata[, c(individual_id, predictors_LME)],
        simplify = FALSE
      ))
    data_predictors_LME[[individual_id]] <-
      as.factor(data_predictors_LME[[individual_id]])
    data_LME <-
      data.frame(data_predictors_LME,
                 response_type,
                 response,
                 response_time)
    data_LME<-data_LME[data_LME$response_time<=x_L,]
    data_LME$predict <- 0
    data_longitudinal <- dplyr::bind_rows(data_LOCF, data_LME)

    mean_response_time <-
      object[[as.character(x_L)]]$model_LME_standardise_time$mean_response_time
    sd_response_time <-
      object[[as.character(x_L)]]$model_LME_standardise_time$sd_response_time
    data_longitudinal$response_time <-
      (data_longitudinal$response_time - mean_response_time) / sd_response_time

    response_predictions <-
      which(data_longitudinal$predict == 1)

    mixoutsamp_longitudinal <-
      mixoutsamp(model = model_LME,
                 newdata = data_longitudinal)


    data_longitudinal <-
      mixoutsamp_longitudinal$preddata[response_predictions,][, c(individual_id,
                                                                  predictors_LME,
                                                                  "response_type",
                                                                  "fitted")]
    data_longitudinal <-
      stats::reshape(
        data_longitudinal,
        timevar = "response_type",
        idvar = c(individual_id, predictors_LME),
        direction = "wide"
      )
    for (name in responses_LME) {
      names(data_longitudinal)[grep(paste0("fitted.", name), names(data_longitudinal))] <-
        name
    }
    if (random_slope_survival == TRUE) {

      if (random_slope_survival == TRUE) {
        if (length(responses_LME)==1){
          slopes_df<-mixoutsamp_longitudinal$random[,c("id","reffresponse_time")]
          slopes_df["reffresponse_time"]<-slopes_df["reffresponse_time"]+model_LME$coefficients$fixed["response_time"]

        }
        if (length(responses_LME)>1){
          slopes_df<-mixoutsamp_longitudinal$random[,c("id",paste0("reffresponse_type",responses_LME,":response_time"))]
          responses_LME_dummy<-paste0("response_type",responses_LME,":response_time")
          responses_LME_dummy[1]<-"response_time"
          for (i in 1:length(responses_LME)){
            slopes_df[,paste0("reffresponse_type",responses_LME[i],":response_time")]<-
              slopes_df[,paste0("reffresponse_type",responses_LME[i],":response_time")]+model_LME$coefficients$fixed[responses_LME_dummy[1]]
            if(i!=1){slopes_df[,paste0("reffresponse_type",responses_LME[i],":response_time")]<-
              slopes_df[,paste0("reffresponse_type",responses_LME[i],":response_time")]+model_LME$coefficients$fixed[responses_LME_dummy[i]]}
          }
        }
        names(slopes_df)<-c("id",paste0(responses_LME,"_slope"))
        data_longitudinal <- dplyr::left_join(data_longitudinal,
                                                  slopes_df,
                                                  by = individual_id)
      }
    }
  }

  if (!is.na(cv_fold)) {
    model_survival <-
      object[[as.character(x_L)]]$model_survival[[as.character(cv_fold)]]
  } else{
    model_survival <- object[[as.character(x_L)]]$model_survival
  }
  if (!inherits(model_survival,c("CauseSpecificCox", "FGR", "coxph"))){
    stop("Class of survival model should be 'CauseSpecificCox','FGR', or 'coxph'")
  }
  if (inherits(model_survival,c("CauseSpecificCox", "FGR"))) {
    data_longitudinal$event_prediction <- as.numeric(
      riskRegression::predictRisk(
        model_survival,
        cause = 1,
        newdata = data_longitudinal,
        times = x_hor,
        ...
      )
    )
  }
  if (inherits(model_survival,"coxph")) {
    data_longitudinal$event_prediction <- as.numeric(
      riskRegression::predictRisk(model_survival, times = x_hor, newdata = data_longitudinal, ...)
    )
  }
  return(data_longitudinal)
}

#' Select individuals in a dataset with a last observation carried forward (LOCF) at a landmark time
#'
#' To fit the LOCF model, all individuals must have at least one
#' non-`NA` entry by landmark time `x_L` for all covariates.
#' This function selects these individuals and removes the other rows.
#'
#' @details Individuals have a LOCF if there is a non-`NA` entry for each of the covariates in
#' `covariates` up until (not including) time `x_L`.
#'
#' @param data_long Data frame with repeated measurements data in long format
#' @template individual_id
#' @template x_L
#' @template covariates
#' @template covariates_time
#' @return List of data frames which correspond to each landmark time `x_L`.
#' Each data frame is an updated version of `data_long` which contains only rows
#' of individuals with a LOCF at age `x_L`, other rows are removed.
#' @author Isobel Barrott \email{isobel.barrott@@gmail.com}
#' @examples
#' library(Landmarking)
#' data(data_repeat_outcomes)
#' data_repeat_outcomes <-
#'   return_ids_with_LOCF(
#'     data_long = data_repeat_outcomes,
#'     individual_id = "id",
#'     covariates =
#'       c("ethnicity", "smoking", "diabetes", "sbp_stnd", "tchdl_stnd"),
#'     covariates_time =
#'       c(rep("response_time_sbp_stnd", 4), "response_time_tchdl_stnd"),
#'     x_L = c(60,61)
#'   )
#' @export



return_ids_with_LOCF <-
  function(data_long,
           individual_id,
           x_L,
           covariates,
           covariates_time) {
    if (!(inherits(data_long,"data.frame"))) {
      stop("data_long should be a data frame")
    }
    if (!(inherits(individual_id,"character"))) {
      stop("individual_id should have class character")
    }
    if (!(inherits(covariates,"character"))) {
      stop("covariates should have class character")
    }
    if (!(inherits(covariates_time,"character"))) {
      stop("covariates_time should have class character")
    }

    for (col in c(covariates,
                  individual_id,
                  covariates_time)) {
      if (!(col %in% names(data_long))) {
        stop(col, " is not a column name in data_long")
      }
      # if(any(is.na(data_long[[col]]))){
      #   stop(col, " contains NA values")
      # }
    }

    if (!(length(covariates_time) %in% c(length(covariates), 1))) {
      stop("Length of covariates_time should be equal to length of covariates or 1")
    }

    if (length(covariates_time) == 1) {
      covariates_time <- rep(covariates_time, times = length(covariates))
    }
    if (!(inherits(x_L,"numeric"))) {
      stop("x_L should have class numeric")
    }

    out_list <- lapply(x_L, function(x_l) {
      #Pick out IDs with one LOCF before Ln for each exposure
      LOCF_IDs_by_variable <-
        lapply(1:length(c(covariates)), function(i) {
          var <- c(covariates)[i]
          time <- c(covariates_time)[i]
          data_var <- data_long[data_long[[time]] <= x_l, ]
          data_var <-
            data_var[!is.na(data_var[[var]]), ]
          data_var <-
            data_var[order(data_var[[time]], decreasing = TRUE), ]
          data_var <-
            data_var[!duplicated(data_var[[individual_id]]), ]
          return(data_var[[individual_id]])
        })
      LOCF_IDs_by_variable <-
        Reduce(intersect, LOCF_IDs_by_variable)
      data_long <-
        data_long[data_long[[individual_id]] %in% LOCF_IDs_by_variable, ]
      data_long
    })
    if (length(x_L) == 1) {
      out_list[[1]]
    } else{
      names(out_list) <- x_L
      out_list
    }
  }

#' Find the risk set for a landmark model (LOCF)
#'
#' This function is a helper function for `fit_LOCF_landmark`.
#'
#' @param data_long Data frame in long format i.e. there may be more than one row per individual
#' @template x_L
#' @template x_hor
#' @template covariates
#' @template covariates_time
#' @template individual_id
#' @template event_time
#' @template event_status
#' @return List with elements corresponding to each landmark time in x_L. Each element is a data frame, containing only those individuals
#' in the risk set at each of the landmark times x_L.
#'
#'
#' @author Isobel Barrott \email{isobel.barrott@@gmail.com}
#' @details This function finds the risk set for each of landmark times in x_L. This means that each of the individuals has a LOCF value for all covariates at the landmark time and
#' has not experienced an event up to (and including) the landmark time.
#' @export

find_LOCF_risk_set <- function(data_long,
                                  x_L,
                               x_hor,
                                  covariates,
                                  covariates_time,
                                  individual_id,
                               event_time,
                               event_status){
  if (!(is.data.frame(data_long) ||
        is.list(data_long))) {
    stop("data_long should be a list or data.frame")
  }
  if (is.data.frame(data_long)) {
    data_long <- lapply(x_L, function(x_l) {
      data_long
    })
    names(data_long) <- x_L
  }
  if (is.list(data_long)) {
    if (!setequal(names(data_long), x_L)) {
      stop("Names of elements in data_long should be landmark ages x_L")
    }
  }


  if (!(inherits(covariates,"character"))) {
    stop("covariates should have class character")
  }
  if (!(inherits(covariates_time,"character"))) {
    stop("covariates_time should have class character")
  }
  if (!(inherits(individual_id,"character"))) {
    stop("individual_id should have class character")
  }
  if (!(inherits(event_time,"character"))) {
    stop("event_time should have class character")
  }
  if (!(inherits(event_status,"character"))) {
    stop("event_status should have class character")
  }

  if (!(inherits(x_L,"numeric"))) {
     stop("'x_L' should be numeric")
  }
  if (!(inherits(x_hor,"numeric"))) {
     stop("'x_hor' should be numeric")
  }

  if (!(length(covariates_time) %in% c(length(covariates), 1))) {
    stop("Length of covariates_time should be equal to length of covariates or 1")
  }

  if (length(covariates_time) == 1) {
    covariates_time <- rep(covariates_time, times = length(covariates))
  }

  data_long_x_L <- lapply(1:length(x_L), function(i) {

    x_l <- x_L[i]
    x_h <- x_hor[i]

    data_long_x_l <- data_long[[as.character(x_l)]]
    for (col in c(covariates,
                  covariates_time,
                  individual_id,
                  event_time,
                  event_status)) {
      if (!(col %in% names(data_long_x_l))) {
        stop(col, " is not a column name in data_long")
      }
      # if(any(is.na(data_long_x_l[[col]]))){
      #   stop(col, " contains NA values")
      # }
    }

    data_long_x_l[[individual_id]] <-
      as.factor(data_long_x_l[[individual_id]])

    #Pull out individuals in the risk set
    data_long_x_l_risk_set <-
      return_ids_with_LOCF(
        data_long = data_long_x_l,
        individual_id = individual_id,
        x_L = x_l,
        covariates = covariates,
        covariates_time = covariates_time
      )
    data_long_x_l_risk_set <-
      data_long_x_l_risk_set[data_long_x_l_risk_set[[event_time]] > x_l,]
    n <-
      length(unique(data_long_x_l[[individual_id]])) - length(unique(data_long_x_l_risk_set[[individual_id]]))

    if (n >= 1) {
      warning(
        n,
        " individuals have been removed from the model building as they are not in the risk set at landmark age ",
        x_l
      )
    }
    data_long_x_l <- data_long_x_l_risk_set

    return(data_long_x_l)
  })
  names(data_long_x_L)<-x_L
  data_long_x_L
}

#' Find the last observation carried forward (LOCF) values for covariates in a dataset
#'
#' This function is a helper function for `fit_LOCF_landmark`.
#'
#' @param data_long Data frame in long format i.e. there may be more than one row per individual
#' @template x_L
#' @template covariates
#' @template covariates_time
#' @param cv_name Character string specifying the column name in `data_long` that indicates cross-validation fold
#' @template individual_id
#' @return List containing `data_longitudinal`, `model_longitudinal`, and `call`.
#'
#' `data_longitudinal` has one row for each individual in `data_long` and
#' contains the LOCF value of `covariates` at the landmark time `x_L`.
#'
#' `model_longitudinal` indicates that the LOCF approach is used.
#'
#' `call` contains the call of the function.
#'
#' @author Isobel Barrott \email{isobel.barrott@@gmail.com}
#' @details This function extracts the LOCF value for each of the `covariates` in `data_long` up to (and including) time `x_L`.
#' @export

fit_LOCF_longitudinal <- function(data_long,
                                  x_L,
                                  covariates,
                                  covariates_time,
                                  cv_name = NA,
                                  individual_id) {
  call <- match.call()
  if (!(inherits(data_long,"data.frame"))) {
    stop("data_long should be a data frame")
  }
  if (!(inherits(x_L,"numeric"))) {
    stop("'x_L' should be numeric")
  }

  for (col in c(covariates,
                covariates_time,
                individual_id)) {
    if (!(col %in% names(data_long))) {
      stop(col, " is not a column name in data_long")
    }
    # if (any(is.na(data_long[[col]]))){
    #   stop(col, " contains NA values")
    # }
  }

  if (!is.na(cv_name)) {
    if (!(cv_name %in% names(data_long))) {
      stop(cv_name, " is not a column name in data_long")
    }
    if (any(is.na(data_long[[cv_name]]))) {
      stop("The column ", cv_name, " contains NA values")
    }
  }

  if (!(length(covariates_time) %in% c(length(covariates), 1))) {
    stop("Length of covariates_time should be equal to length of covariates or 1")
  }

  if (length(covariates_time) == 1) {
    covariates_time <- rep(covariates_time, times = length(covariates))
  }

  if (dim(
    return_ids_with_LOCF(
      data_long = data_long,
      individual_id = individual_id,
      x_L = x_L,
      covariates = covariates,
      covariates_time = covariates_time
    )
  )[1] != dim(data_long)[1]) {
    stop(
      paste0(
        "data_long contains individuals that do not have a LOCF for all covariates at landmark age ",
        x_L,
        ".
                  Use function return_ids_with_LOCF to remove these individuals from the dataset data_long."
      )
    )
  }

  data_long[[individual_id]] <-
    as.factor(data_long[[individual_id]])
  data_LOCF <- data_long

  #Pick out LOCF for each covariate
  LOCF_values_by_variable <-
    lapply(1:length(covariates), function(x) {
      return_LOCF_by_variable(
        data_long = data_LOCF,
        i = x,
        covariates = covariates,
        covariates_time = covariates_time,
        individual_id = individual_id,
        x_L = x_L
      )
    })
  data_LOCF <- Reduce(merge, LOCF_values_by_variable)
  data_LOCF <-
    data_LOCF[match(unique(data_long[[individual_id]][data_long[[individual_id]] %in% data_LOCF[[individual_id]]]), data_LOCF[[individual_id]]), ]
  if (!is.na(cv_name)) {
    data_LOCF <-
      dplyr::left_join(data_LOCF, unique(data_long[c(individual_id, cv_name)]), by =
                         individual_id)
  }
  data_LOCF <-
    data_LOCF[, order(match(names(data_LOCF), names(data_long)))]
  data_LOCF <-
    data_LOCF[order(match(data_LOCF[[individual_id]], data_long[[individual_id]])), ]
  rownames(data_LOCF) <- NULL

  list(
    data_longitudinal = data_LOCF,
    model_longitudinal = "LOCF",
    call = call
  )
}

#' Fit a landmark model using a last observation carried forward (LOCF) method for the longitudinal data
#'
#' This function performs the two-stage landmarking analysis.
#'
#' @param data_long Data frame or list of data frames each corresponding to a landmark age `x_L` (each element of the list must be named the value of `x_L` it corresponds to).
#' Each data frame contains repeat measurements data and time-to-event data in long format.
#' @template x_L
#' @template x_hor
#' @template event_status
#' @template event_time
#' @param k Integer specifying the number of folds for cross-validation. An alternative to setting parameter `cross_validation_df` for performing cross-validation;
#' if both are missing no cross-validation is used.
#' @template cross_validation_df
#' @template b
#' @template covariates
#' @template covariates_time
#' @template individual_id
#' @template survival_submodel
#' @return List containing containing information about the landmark model at each of the landmark times.
#' Each element of this list is named the corresponding landmark time, and is itself a list containing elements:
#' `data`, `model_longitudinal`, `model_survival`, and `prediction_error`.
#'
#' `data`  has one row for each individual in the risk set at `x_L` and
#' contains the value of the `covariates` at the landmark time `x_L` using the LOCF approach. It also includes the predicted
#' probability that the event of interest has occurred by time \code{x_hor}, labelled as \code{"event_prediction"}.
#' There is one row for each individual.
#'
#' `model_longitudinal` indicates that the longitudinal approach is LOCF.
#'
#' `model_survival` contains the outputs from the function used to fit the survival submodel, including the estimated parameters of the model.
#' For a model using cross-validation, `model_survival` contains a list of outputs with each
#' element in the list corresponding to a different cross-validation fold. For more information on how the survival model is fitted
#' please see `?fit_survival_model` which is a function used within `fit_LOCF_landmark`.
#'
#' `prediction_error` contains a list indicating the c-index and Brier score at time `x_hor` and their standard errors if parameter `b` is used.
#' For more information on how the prediction error is calculated
#' please see `?get_model_assessment` which is the function used to do this within `fit_LOCF_landmark`.
#'
#' @details Firstly, this function selects the individuals in the risk set at the landmark time \code{x_L}.
#' Specifically, the individuals in the risk set are those that have entered the study before the landmark time \code{x_L}
#' (there is at least one observation for each of the \code{predictors_LME} and \code{random_effects} on or before \code{x_L}) and
#' exited the study after the landmark age (\code{event_time}
#' is greater than \code{x_L}).
#'
#' Secondly, if the option to use cross validation
#' is selected (using either parameter `k` or `cross_validation_df`), then an extra column `cross_validation_number` is added with the
#' cross-validation folds. If parameter `k` is used, then the function `add_cv_number`
#' randomly assigns these folds. For more details on this function see `?add_cv_number`.
#' If the parameter `cross_validation_df` is used, then the folds specified in this data frame are added.
#' If cross-validation is not selected then the landmark model is
#' fit to the entire group of individuals in the risk set (this is both the training and test dataset).
#'
#' Thirdly, the landmark model is then fit to each of the training datasets. There are two parts to fitting the landmark model: using the longitudinal data and using the survival data.
#' Using the longitudinal data is the first stage and is performed using `fit_LOCF_longitudinal`. See `?fit_LOCF_longitudinal` more for information about this function.
#' This function censors the
#' individuals at the time horizon `x_L` and fits the survival model. Using the survival data is the second stage and is performed using `fit_survival_model`. See `?fit_survival_model` more for information about this function.
#'
#' Fourthly, the performance of the model is then assessed on the set of predictions
#' from the entire set of individuals in the risk set by calculating Brier score and C-index.
#' This is performed using `get_model_assessment`. See `?get_model_assessment` more for information about this function.
#'
#' @author Isobel Barrott \email{isobel.barrott@@gmail.com}
#' @examples
#' library(Landmarking)
#' data(data_repeat_outcomes)
#' data_model_landmark_LOCF <-
#'    fit_LOCF_landmark(
#'      data_long = data_repeat_outcomes,
#'      x_L = c(60, 61),
#'      x_hor = c(65, 66),
#'      covariates =
#'        c("ethnicity", "smoking", "diabetes", "sbp_stnd", "tchdl_stnd"),
#'      covariates_time =
#'        c(rep("response_time_sbp_stnd", 4), "response_time_tchdl_stnd"),
#'      k = 10,
#'      individual_id = "id",
#'      event_time = "event_time",
#'      event_status = "event_status",
#'      survival_submodel = "cause_specific"
#'    )
#' @importFrom stats as.formula
#' @importFrom survival Surv
#' @importFrom survival coxph
#' @importFrom prodlim Hist
#' @export


fit_LOCF_landmark <- function(data_long,
                              x_L,
                              x_hor,
                              covariates,
                              covariates_time,
                              k,
                              cross_validation_df,
                              individual_id,
                              event_time,
                              event_status,
                              survival_submodel = c("standard_cox", "cause_specific", "fine_gray"),
                              b) {
  call <- match.call()

  survival_submodel <- match.arg(survival_submodel)

  #Checks
  if (missing(k)) {
    k_add <- FALSE
  }
  else{
    k_add <- TRUE
    if (!(is.numeric(k))) {
      stop("k should be numeric")
    }
  }

  if (missing(cross_validation_df)) {
    cross_validation_df_add <- FALSE
  }
  else{
    cross_validation_df_add <- TRUE
    if (inherits(cross_validation_df, "list")) {
      if (!all(x_L %in% names(cross_validation_df))) {
        stop(
          "The names of elements in cross_validation_df list should be the landmark times in x_L"
        )
      }
      if (any(Reduce("c", lapply(cross_validation_df, function(x) {
        any(duplicated(dplyr::distinct(x[, c(individual_id, "cross_validation_number")])[, individual_id]))
      })))) {
        stop("Cross validation folds should be the same for the same individual")
      }
    }
    else if (inherits(cross_validation_df,"data.frame")) {
      if (any(duplicated(dplyr::distinct(cross_validation_df[, c(individual_id, "cross_validation_number")])[, individual_id]))) {
        stop("Cross validation folds should be the same for the same individual")
      }
      cross_validation_df<-list(cross_validation_df)
      names(cross_validation_df)<-x_L
    }
    else{
      stop("cross_validation_df should be either a data frame or a list")
    }
  }
  if (k_add == TRUE &&
      cross_validation_df_add == TRUE) {
    stop("Either use parameter k or cross_validation_df but not both")
  }
  if (k_add == FALSE &&
      cross_validation_df_add == FALSE) {
    cv_name <- NA
  } else{
    cv_name <- "cross_validation_number"
  }

  if (!(length(covariates_time) %in% c(length(covariates), 1))) {
    stop("Length of covariates_time should be equal to length of covariates or 1")
  }
  if (length(covariates_time) == 1) {
    covariates_time <- rep(covariates_time, times = length(covariates))
  }

  if (length(x_L) != length(x_hor)) {
    stop("Length of x_L should be the same as length of x_hor")
  }
  if (!(is.data.frame(data_long) ||
        is.list(data_long))) {
    stop("data_long should be a list or data.frame")
  }
  if (is.data.frame(data_long)) {
    data_long <- lapply(x_L, function(x_l) {
      data_long
    })
    names(data_long) <- x_L
  }
  if (is.list(data_long)) {
    if (!setequal(names(data_long), x_L)) {
      stop("Names of elements in data_long should be landmark ages x_L")
    }
  }

  if (missing(b)) {
    b <- NA
  }
  #Find risk set
  data_long_x_L<-find_LOCF_risk_set(data_long=data_long,
                     x_L=x_L,
                     x_hor=x_hor,
                     covariates=covariates,
                     covariates_time=covariates_time,
                     individual_id=individual_id,
                     event_time=event_time,
                     event_status=event_status)

  #Add cross-validation folds
  if (cross_validation_df_add == TRUE) {
    for (x_l in x_L){
      data_long_x_L[[as.character(x_l)]]<-
        dplyr::left_join(data_long_x_L[[as.character(x_l)]], cross_validation_df[[as.character(x_l)]][,c(individual_id,"cross_validation_number")],by=individual_id)
    }
    if(any(is.na(data_long_x_L[[as.character(x_l)]][,"cross_validation_number"]))){stop("Cross validation number not defined for all in individual_id")}
  }

  if (k_add == TRUE) {
    data_long_x_L_cv <-
      add_cv_number(
        data_long = Reduce("rbind", data_long_x_L),
        individual_id = individual_id,
        k = k
      )
    data_long_x_L <-
      lapply(data_long_x_L, function(x) {
        dplyr::left_join(x, dplyr::distinct(data_long_x_L_cv[, c(individual_id, "cross_validation_number")]), by =
                           individual_id)
      })
  }

  #Fit each landmark model
  out <- lapply(1:length(x_L), function(i) {
    x_l <- x_L[i]
    x_h <- x_hor[i]

    data_long <- data_long_x_L[[as.character(x_l)]]

    message("Fitting longitudinal submodel, landmark age ", x_l)
    data_model_longitudinal <-
      fit_LOCF_longitudinal(
        data_long = data_long,
        x_L = x_l,
        covariates = covariates,
        covariates_time =
          covariates_time,
        cv_name = cv_name,
        individual_id =
          individual_id
      )

    message("Complete, landmark age ", x_l)

    data_events <-
      dplyr::distinct(data_long[, c(individual_id, event_status, event_time)])
    data_longitudinal <-
      dplyr::left_join(data_model_longitudinal$data_longitudinal,
                       data_events,
                       by = individual_id)

    message("Fitting survival submodel, landmark age ", x_l)
    data_model_survival <- fit_survival_model(
      data = data_longitudinal,
      individual_id = individual_id,
      cv_name = cv_name,
      covariates = covariates,
      event_time = event_time,
      event_status = event_status,
      survival_submodel = survival_submodel,
      x_hor = x_h
    )
    message("Complete, landmark age ", x_l)

    data_events <-
      dplyr::left_join(data_events,
                       data_model_survival$data_survival[,c(individual_id,"event_prediction")],
                       by = individual_id)

    prediction_error <-
      get_model_assessment(
        data = data_model_survival$data_survival,
        individual_id = individual_id,
        event_prediction = "event_prediction",
        event_status = event_status,
        event_time = event_time,
        x_hor = x_h,
        b = b
      )

    list(
      data = data_model_survival$data_survival,
      model_longitudinal = data_model_longitudinal$model_longitudinal,
      model_survival = data_model_survival$model_survival,
      prediction_error = prediction_error,
      call = call
    )
  })
  names(out) <- x_L
  class(out) <- "landmark"
  out
}

set.seed(1)
data(data_repeat_outcomes)
data_repeat_outcomes <-
  find_LOCF_risk_set(
    data_long = data_repeat_outcomes,
    individual_id = "id",
    event_time = "event_time",
    event_status = "event_status",
    covariates = c("ethnicity", "smoking", "diabetes", "sbp_stnd", "tchdl_stnd"),
    covariates_time = c(rep("response_time_sbp_stnd", 4), "response_time_tchdl_stnd"),
    x_L = c(60,61),
    x_hor= c(65,66)
  )
data_model_landmark_LOCF <-
  fit_LOCF_landmark(
    data_long = data_repeat_outcomes,
    x_L = c(60, 61),
    x_hor = c(65, 66),
    covariates = c("ethnicity", "smoking", "diabetes", "sbp_stnd", "tchdl_stnd"),
    covariates_time = c(rep("response_time_sbp_stnd", 4), "response_time_tchdl_stnd"),
    k = 10,
    individual_id = "id",
    event_time = "event_time",
    event_status = "event_status",
    survival_submodel = "cause_specific"
  )
cross_validation_list <- lapply(data_model_landmark_LOCF, "[[", i = 1)
data_model_landmark_LME <-
  fit_LME_landmark(
    data_long = data_repeat_outcomes[["60"]],
    x_L = c(60),
    x_hor = c(65),
    cross_validation_df =
      cross_validation_list,
    predictors_LME = c("ethnicity", "smoking", "diabetes"),
    predictors_LME_time =
      "response_time_sbp_stnd",
    responses_LME = c("sbp_stnd", "tchdl_stnd"),
    responses_LME_time = c("response_time_sbp_stnd", "response_time_tchdl_stnd"),
    individual_id = "id",
    standardise_time = TRUE,
    lme_control = nlme::lmeControl(maxIter = 100, msMaxIter = 100),
    event_time = "event_time",
    event_status = "event_status",
    survival_submodel = "cause_specific"
  )
save_file <- function(code) {
  path <- tempfile(fileext = ".RDS")
  saveRDS(code,file = path)
  path
}

test_that("fitting landmark (LOCF)", {
  expect_snapshot_file(save_file(
    data_model_landmark_LOCF), "out_data_model_landmark_LOCF.RDS",cran = FALSE)
})

test_that("fitting landmark (LME)", {
  expect_snapshot_file(save_file(
    data_model_landmark_LME), "out_data_model_landmark_LME.RDS",cran = FALSE)
})

return_LOCF_by_variable <- function(data_long,
                                    i,
                                    covariates,
                                    covariates_time,
                                    individual_id,
                                    x_L) {
  var <- covariates[i]
  time <- covariates_time[i]
  data_var <-
    data_long[data_long[[time]] <= x_L, ][, c(individual_id, var, time)]
  data_var <-
    data_var[!is.na(data_var[[var]]), ]#removes NA values
  data_var <-
    data_var[order(data_var[[time]], decreasing = TRUE),]
  data_var <-
    data_var[!duplicated(data_var[[individual_id]]),]
  return(data_var[, c(individual_id, var)])
}

#' Assign a k-fold cross-validation number
#'
#' Randomly assigns a k-fold cross-validation number to each individual in a dataset.
#'
#' @param data_long Data frame in long format i.e. there may be more than one row per individual
#' @template individual_id
#' @param k Integer specifying the number of folds for cross-validation.
#' @param seed The value of the seed (default is 1)
#' @return Data frame `data_long` updated to contain a new column `cross_validation_number`
#' indicating the fold to which the individual has been assigned.
#' @author Isobel Barrott \email{isobel.barrott@@gmail.com}
#' @examples
#' data(data_repeat_outcomes)
#' data_repeat_outcomes <- add_cv_number(data_long = data_repeat_outcomes,
#'                                       individual_id = "id",
#'                                       k = 10)
#' @details This function randomly divides the \code{n} individual IDs
#' into \code{k} groups, each with \code{n/k} members (or as close to this number as possible).
#' @export



add_cv_number <- function(data_long, individual_id, k, seed = 1) {
  set.seed(seed)
  if (!(inherits(data_long,"data.frame"))) {
    stop("data_long should be a data frame")
  }
  if (!(inherits(individual_id,"character"))) {
    stop("individual_id should have class character")
  }

  if (!(individual_id %in% colnames(data_long))) {
    stop("individual_id should be a column name in data_long")
  }
  if(any(is.na(data_long[[individual_id]]))){
    stop(individual_id, " contains NA values")
  }
  if (!(inherits(k,"numeric"))) {
    stop("k should have class numeric")
  }
  ids <- data_long[[individual_id]]
  unique_ids <- unique(ids)
  n <- length(unique_ids)
  sample_size <- rep(round(n / k, 0), k)
  diff <- n - round(n / k, 0) * k
  if (diff != 0) {
    sample_size[1:abs(diff)] <- sample_size[1:abs(diff)] + sign(diff)
  }
  sample_size_end <- cumsum(sample_size)
  sample_size_start <- c(1, cumsum(sample_size)[-k] + 1)
  samples <- sample(1:n, size = n)
  df <- data.frame(cross_validation_number = rep(NA, n))
  df[[individual_id]] <- methods::as(unique_ids, class(ids))
  for (i in 1:k) {
    df[samples[sample_size_start[i]:sample_size_end[i]], "cross_validation_number"] <-
      i
  }
  df <- dplyr::left_join(data_long, df, by = individual_id)
  df
}

#' Simulated repeat measurement and time-to-event data
#'
#' @description A simulated dataset which is a combination of longitudinal (repeat measurement) data and
#' time-to-event data. This dataset contains simulated data
#' from 3000 patients.
#'
#' The longitudinal (repeat measurement) data is formed using an LME model,
#' whose parameters were based on CVD risk assessments
#' recorded at primary care practices in New Zealand.
#'
#' A LME model was fitted to this dataset and values of \code{sbp_stnd}
#' and \code{tchdl_stnd} were estimated at landmark age 60. These values (along with the other baseline covariates)
#' were used to simulate time-to-event data from a
#' cause specific model with parameters based on CVD events of patients
#' at primary care practices in New Zealand.
#'
#' @format A dataset with 9031 rows and 12 columns:
#' \describe{
#'   \item{id}{Patient ID}
#'   \item{smoking}{Smoking status, 0 indicates the patient has never smoked, 1 indicates the patient has quit smoking, and 2 indicates the patient is a current smoker}
#'   \item{diabetes}{Diabetes status, 0 indicates the patient is not diagnosed with diabetes, and 1 indicates the patient is diagnosed with diabetes}
#'   \item{ethnicity}{Ethnicity, one of five ethnicities}
#'   \item{dob}{Date of birth}
#'   \item{index}{An index indicating assessment number for a patient}
#'   \item{sbp_stnd}{Standardised systolic blood pressure}
#'   \item{tchdl_stnd}{Standardised total cholesterol to HDL ratio}
#'   \item{response_time_tchdl_stnd}{Age that total cholesterol to HDL ratio was recorded}
#'   \item{response_time_sbp_stnd}{Age that systolic blood pressure was recorded, this is the same as the date that the fixed measures were recorded}
#'   \item{event_status}{Event status, 0 indicates censoring, 1 indicates CVD event, and 2 indicates death from other causes}
#'   \item{event_time}{Event time}
#' }
#'
"data_repeat_outcomes"

#' Calculate point estimates from a linear mixed effects (LME) model for new data
#'
#' This function allows the user to make out-of-sample predictions from an LME model.
#'
#' @param model Object of class `lme` containing the fitted LME model
#' @param newdata Data frame containing data for which to make predictions.
#' The response variable should be set to NA for the rows of the data the user wishes to make predictions for.
#' The columns in the data should have the same names as those used to fit the model.
#' The variables should also be of the same type as in the data used to fit the mixed model (numeric, factor etc).
#' @return List containing `preddata` and `random`. Data frame `preddata` is a version of `newdata` updated to contain columns corresponding to the fixed effects values (`fixed`),
#' random effects values (`random`), and fitted values (`fitted`).
#' Data frame `random` contains the values of random effects components for each individual.
#' @author This code was originally written by Ruth Keogh (London School of Hygiene and Tropical Medicine) which can be viewed at
#' github.com/ruthkeogh/landmark_CF. There have been further contributions from Jessica Barrett (MRC Biostatistics Unit, University of Cambridge),
#' David Stevens (University of Liverpool), and Mike Sweeting (University of Leicester).
#' @export


mixoutsamp <- function(model, newdata) {
  n = dim(newdata)[1]

  #----error message
  if (!(inherits(model,"lme"))) {
    stop("Error: model is not of type lme")
  }
  #----end of error message

  #----error message
  if (is.null(newdata) == T) {
    stop("Error: no data was provided in newdata")
  }
  #----end of error message

  #name of the id (group) variable
  id.name = names(model$groups)

  #name of the response variable
  y.name = as.character(attr(model$terms, "variables")[[2]])

  #name of variable which distinguished between different response variables (for multivariate situation)
  if (is.null(model$modelStruct$varStruct) == 0) {
    ytype.name = as.character(stats::formula(model$modelStruct$varStruct)[[2]][[3]])
  } else{
    ytype.name = NULL
  }


  re.names = attr(model$modelStruct$reStr[[1]], "Dimnames")[[1]]

  #type of within-person correlation structure
  if (is.null(attributes(model$modelStruct$corStruct)$class[1]) == 1) {
    corr.struct.type = "indep"
  } else {
    corr.struct.type = attributes(model$modelStruct$corStruct)$class[1]
  }

  #----error message

  data.names <-
    unique(c(all.vars(model$terms), all.vars(
      attr(model$modelStruct$reStruct$patid, "formula")
    ), all.vars(
      attr(model$modelStruct$varStruct, "formula")
    )))
  #names of all variables used in any part of the model

  # if(is.null(ytype.name)==1){
  # data.names=c(id.name,
  #   as.character(attributes(model$terms)$variables)[-1],#response variable and variables with fixed effects
  #   attributes(getVarCov(model))$dimnames[[1]])#variables with random effects
  # }else{
  #     data.names=c(id.name,
  #                  as.character(attributes(model$terms)$variables)[-1],#response variable and variables with fixed effects
  #                  attributes(getVarCov(model))$dimnames[[1]]#variables with random effects
  #                  )}
  #
  # data.names=data.names[data.names!="(Intercept)"]
  # data.names=unique(data.names)
  #data.names=data.names[-which(sapply(1:length(data.names),function(v)grepl(":",data.names[v]))==T)]
  #this omits interactions between variables already listed (this is necessary to avoid an error occurring in the error message where I check whether newdata contains all the variables in the model)

  #check that newdata contains all variable names in data.names

  if (sum(names(newdata) %in% data.names) != length(data.names)) {
    stop("Error: newdata does not contain the variables used in the model")
  }

  #----end of error message

  #----error message
  for (v in 1:length(data.names)) {
    if (sum(is.na(newdata[, v])) == n) {
      stop(paste0(
        "Error: the variable ''",
        data.names[v],
        "'' has no non-missing values"
      ))
    }
  }
  #----end of error message

  #-------------------------------------------------------------
  #model parameters

  coef.fixed = matrix(model$coef$fixed,
                      nrow = length(model$coef$fixed),
                      ncol = 1) #vector of fixed coefficients
  G = as.matrix(nlme::getVarCov(model))  #random effect variance-covariance matrix
  #residual variance: this is a single value if the variance is homoscedastic but a vector of values in cases of heteroscedasticity (e.g. when we have a multivariable outcome)

  if (is.null(attributes(model$modelStruct$varStruct)$weights) == 1) {
    resid.var = (model$sigma) ^ 2
  } else if (is.null(attributes(model$modelStruct$varStruct)$weights) ==
             0) {
    #resid.var= matrix((model$sigma/unique(attributes(model$modelStruct$varStruct)$weights))^2,nrow=1,byrow=TRUE)
    resid.var = (model$sigma * stats::coef(
      nlme::varFunc(model$modelStruct$varStruct),
      unconstrained = F,
      allCoef = T
    )) ^ 2
  }

  observed <- !logical(dim(newdata)[1])
  for (v in 1:length(data.names)) {
    if (data.names[v] == y.name)
      next
    observed = observed & (!is.na(newdata[, data.names[v]]))
  }
  #-------------------------------------------------------------
  #the observed data (rows of newdata where the response y is observed)
  newdata <- newdata[observed, ]
  ids <-
    unique(newdata[, id.name])  #vector of unique id numbers (aka the grouping variable)
  is.y.obs = !is.na(newdata[, y.name]) #indicator of whether y is observed (1: no, 0: yes)


  x.mat = stats::model.matrix(model, data = newdata[is.y.obs, ])  #fixed effects design matrix
  z.mat = stats::model.matrix(stats::formula(model$modelStruct$reStr)[[1]], data =
                                newdata[is.y.obs, ]) #random effects design matrix
  y = newdata[, y.name][is.y.obs] #the outcome
  xb = x.mat %*% coef.fixed   #fitted values for the fixed part of the model
  if (is.null(model$modelStruct$varStruct) == 0) {
    ytype = newdata[, ytype.name][is.y.obs] #response variable type (for multivariate situation)
  }

  obs.ids = newdata[, id.name][is.y.obs] #id numbers

  #vector of times used in the correlation structure
  if (corr.struct.type != "indep") {
    vec.corr.times = stats::model.matrix(stats::as.formula(paste(
      "~", stats::formula(model$modelStruct$corStruct)[[2]][[2]]
    )), data = newdata[is.y.obs == 0, ])[, -1]
  }

  #-------------------------------------------------------------
  #within-person correlation parameters

  num.raneff = dim(z.mat)[2]   #number of random effects
  #parameters associated with the within-person correlation
  if (corr.struct.type == "corExp") {
    range = stats::coef(model$modelStruct$corStruct, unconstrained = F)
  } else if (corr.struct.type == "corLin") {
    range = stats::coef(model$modelStruct$corStruct, unconstrained = F)
  } else if (corr.struct.type == "corGaus") {
    range = stats::coef(model$modelStruct$corStruct, unconstrained = F)
  } else if (corr.struct.type == "corRatio") {
    range = stats::coef(model$modelStruct$corStruct, unconstrained = F)
  } else if (corr.struct.type == "corSpher") {
    range = stats::coef(model$modelStruct$corStruct, unconstrained = F)
  } else if (corr.struct.type == "corCompSymm") {
    Rho = stats::coef(model$modelStruct$corStruct, unconstrained = F)
  } else if (corr.struct.type == "corAR1" |
             corr.struct.type == "corCAR1") {
    Phi = stats::coef(model$modelStruct$corStruct, unconstrained = F)
  }


  #-------------------------------------------------------------
  #the data for which predictions are required - this is for all individuals in newdata

  xstar.mat = stats::model.matrix(model,
                                  stats::model.frame( ~ ., newdata, na.action = stats::na.pass)) #fixed effects design matrix
  zstar.mat = stats::model.matrix(stats::formula(model$modelStruct$reStr)[[1]], data =
                                    newdata) #random effects design matrix
  xbstar <-
    xstar.mat  %*% coef.fixed   #fitted values for the fixed part of the model

  pred.ids = newdata[, id.name]  #id numbers
  ids = unique(pred.ids) #unique id numbers

  #-------------------------------------------------------------
  #loop over individuals to give random effect part of the fitted values and individual random effects

  reffects = matrix(nrow = length(pred.ids), ncol = 1)
  reffects.individual = matrix(nrow = length(ids), ncol = num.raneff)
  vec.zeros = as.vector(rep(0, num.raneff))

  split.obs.ids <-
    split(1:length(obs.ids), newdata[is.y.obs, id.name]) #list of ids with rows that contain observed values
  split.pred.ids <-
    split(1:length(pred.ids), newdata[, id.name]) #list of ids with all values

  # pb<-txtProgressBar(0,length(split.obs.ids),style=3)
  for (i in 1:length(split.obs.ids)) {
    #  setTxtProgressBar(pb,i)
    oids.list <- split.obs.ids[[i]]
    pids.list <- split.pred.ids[[i]]



    if (corr.struct.type != "indep") {
      t.rep = matrix(
        rep(vec.corr.times[obs.ids %in% ids[i]], length(vec.corr.times[obs.ids %in%
                                                                         ids[i]])),
        nrow = length(vec.corr.times[obs.ids %in% ids[i]]),
        ncol = length(vec.corr.times[obs.ids %in% ids[i]])
      )
      distances = abs(t.rep - t(t.rep))
      distances.2 = t.rep - t(t.rep)
    }
    if (corr.struct.type == "indep") {
      corr.mat = diag(1, length(oids.list))
    } else if (corr.struct.type == "corExp") {
      corr.mat = exp(-distances / range)
    } else if (corr.struct.type == "corLin") {
      corr.mat = ifelse(distances < range, 1 - distances / range, 0)
    } else if (corr.struct.type == "corGaus") {
      corr.mat = exp(-(distances / range) ^ 2)
    } else if (corr.struct.type == "corRatio") {
      corr.mat = 1 / (1 + (distances / range) ^ 2)
    } else if (corr.struct.type == "corSpher") {
      corr.mat = ifelse(distances < range,
                        1 - 1.5 * (distances / range) + 0.5 * (distances / range) ^ 3,
                        0)
    } else if (corr.struct.type == "corCompSymm") {
      corr.mat = matrix(Rho, nrow = nrow(t.rep), ncol = ncol(t.rep))
    } else if (corr.struct.type == "corAR1" |
               corr.struct.type == "corCAR1") {
      corr.mat = Phi ^ distances
    } #else if(corr.struct.type=="corARMA" & p==1 & q==0){
    #   corr.mat=Phi^distances
    # }



    if (is.null(model$modelStruct$varStruct) == 1) {
      resid.var.mat = matrix(resid.var,
                             nrow = dim(corr.mat)[1],
                             ncol = dim(corr.mat)[1])
    } else if (is.null(model$modelStruct$varStruct) == 0) {
      resid.var.mat <-
        as.matrix(diag(
          resid.var[as.character(ytype[oids.list])],
          nrow = dim(corr.mat)[1],
          ncol = dim(corr.mat)[1]
        ))
    }
    z.mat.oids <- z.mat[oids.list, , drop = FALSE]
    t.z.mat.oids <- t(z.mat.oids)
    reffects.individual[i, ] <- (G %*% t.z.mat.oids) %*%
      solve((z.mat.oids %*% G %*% t.z.mat.oids + resid.var.mat * corr.mat)) %*%
      (y[oids.list] - xb[oids.list])
    reffects[pids.list] <-
      zstar.mat[pids.list, , drop = FALSE] %*% t(reffects.individual[i, , drop = FALSE])
    #the use of drop=FALSE in z.mat on the first and second lines handles the annoying fact that when we have a matrix with one row, R automatically converts it into a vector and we get non-conformability issues.
  }

  #-------------------------------------------------------------
  #calculate fitted values

  fitted = xbstar + reffects

  #--------------
  #things to output

  preddata = cbind(newdata, xbstar, reffects, fitted)
  names(preddata) = c(names(newdata), "fixed", "random", "fitted")
  random = data.frame(names(split.obs.ids), reffects.individual)
  names(random) = c(id.name,  paste0("reff", re.names))

  list(preddata = preddata, random = random)

}

#' Create a calibration plot
#'
#' Creates a calibration plot for the landmark model fitted by `fit_LME_landmark_model` or `fit_LOCF_landmark_model`.
#' This function plots the observed frequencies of the event of interest against the predicted probabilities of the event of interest.
#'
#' @param x Object inheriting the class `landmark`, this should be the output from either `fit_LME_landmark_model` or `fit_LOCF_landmark_model`. It should contain a list
#' of landmark models corresponding to different landmark times `x_L`.
#' @param x_L Numeric specifying the landmark time. This indicates which landmark model in `x` to use.
#' @param n Numeric specifying the number of bins to use.
#' @param x_lims Vector of length 2 specifying the limits of the x axes
#' @param y_lims Vector of length 2 specifying the limits of the y axes
#' @param \dots Arguments passed to `ggplot2::labs` to modify axis, legend, and plot labels
#' @return Calibration plot showing the value of predicted probabilities against observed frequencies, with a `y=x` line.
#' @details This function bins the predicted probabilities of the event of interest into `n` bins. The event of interest is the event with
#' `event_status=1` when fitting the landmark model. For each of the `n` sets of individuals, the Aalen-Johansen estimator is fit to that set
#' and used to calculate the risk of an event at the horizon time. The predictions (from the landmark model) and the observed frequencies
#' (from the Aalen-Johansen estimator) are plotted against each other. For a perfect prediction model, the points will be plotted along the y=x line.
#' @examples
#' library(Landmarking)
#' data(data_repeat_outcomes)
#' data_model_landmark_LOCF <-
#'   fit_LOCF_landmark(
#'     data_long = data_repeat_outcomes,
#'     x_L = c(60, 61),
#'     x_hor = c(65, 66),
#'     covariates =
#'       c("ethnicity", "smoking", "diabetes", "sbp_stnd", "tchdl_stnd"),
#'     covariates_time =
#'       c(rep("response_time_sbp_stnd", 4), "response_time_tchdl_stnd"),
#'     k = 10,
#'     individual_id = "id",
#'     event_time = "event_time",
#'     event_status = "event_status",
#'     survival_submodel = "cause_specific"
#'   )
#'  plot(x=data_model_landmark_LOCF,x_L=60,n=5)
#'  plot(x=data_model_landmark_LOCF,x_L=61,n=5)
#' @export


plot.landmark <- function(x, x_L, n, x_lims, y_lims, ...) {
  if (!inherits(x,"landmark")) {
    stop("x must have class 'landmark'")
  }
  if (!inherits(x_L,"numeric")) {
    stop("x_L should be numeric")
  }
  if (!inherits(n,"numeric")) {
    stop("n must have class numeric")
  }
  if (!(as.character(x_L) %in% names(x))) {
    stop("x_L must be the name of an element in 'x'")
  }
  x <- x[[as.character(x_L)]]
  event_status <- x$call$event_status

  strata <- survival::strata
  actual <- c()
  predicted <- c()
  data_survival <- x$data
  event_time <- x$call$event_time
  data_survival[["event_time"]] <- data_survival[[event_time]]
  events <- unique(x[["data"]][[event_status]])

  data_survival$quantile <-
    .bincode(
      data_survival$event_prediction,
      breaks = stats::quantile(data_survival$event_prediction, seq(0, 1, by =
                                                                     1 / n)),
      include.lowest = TRUE
    )
  if (length(table(data_survival$quantile)) != n) {
    stop("Not enough data for number of quantiles, select lower value of n")
  }
  trans <- mstate::trans.comprisk(max(events), 1:max(events))
  data_survival <- do.call("rbind", lapply(1:max(events), function(i) {
    data_survival_i <-
      data.frame(data_survival, trans = i, status = as.numeric(+(data_survival[[event_status]] ==
                                                                   i)))
    if (all(data_survival[["status"]] == i) == 0) {
      stop(paste0("Not enough events (for event ", i, ")"))
    }
    data_survival_i
  }))
  for (i in 1:n) {
    data_i <- data_survival[data_survival$quantile == i, ]
    coxph_i <-
      survival::coxph(
        survival::Surv(event_time, status) ~ strata(trans),
        data = data_i,
        method = "breslow"
      )
    newdata <- data.frame(trans = 1:max(events))
    msfit_i <-
      mstate::msfit(object = coxph_i,
                    newdata = newdata,
                    trans = trans)
    probtrans_i <-
      mstate::probtrans(msfit_i, predt = 0, method = "aalen")[[1]]

    actual <- c(actual, probtrans_i[dim(probtrans_i)[1], "pstate2"])
    predicted <- c(predicted, mean(data_i[, "event_prediction"]))
  }
  calibration_plot_df <- data.frame(actual, predicted)

  if (missing(x_lims)) {
    x_lims <- c(0, 1)
  }
  if (missing(y_lims)) {
    y_lims <- c(0, 1)
  }

  ggplot2::ggplot(calibration_plot_df, ggplot2::aes(x = predicted, y = actual)) +
    ggplot2::geom_point(...) + ggplot2::geom_line()  +
    # ggplot2::labs(x = "Predicted probability", y = "Observed frequency") +
    ggplot2::geom_abline(intercept = 0,
                         slope = 1,
                         linetype = "dashed") +
    ggplot2::scale_x_continuous(limits = c(x_lims[1], x_lims[2])) +
    ggplot2::scale_y_continuous(limits = c(y_lims[1], y_lims[2]))
}

#' Find the risk set for a landmark model (LME)
#'
#' This function is a helper function for `fit_LME_landmark`.
#'
#' @param data_long Data frame in long format i.e. there may be more than one row per individual
#' @template x_L
#' @template x_hor
#' @template predictors_LME
#' @template predictors_LME_time
#' @template responses_LME
#' @template responses_LME_time
#' @template individual_id
#' @template event_time
#' @template event_status
#' @return List with elements corresponding to each landmark time in x_L. Each element is a data frame, containing only those individuals
#' in the risk set at each of the landmark times x_L.
#'
#'
#' @author Isobel Barrott \email{isobel.barrott@@gmail.com}
#' @details This function finds the risk set for each of landmark times in x_L. This means that each of the individuals has a LME value for all predictors_LME at the landmark time and
#' has not experienced an event up to (and including) the landmark time.
#' @export

find_LME_risk_set <- function(data_long,
                               x_L,
                               x_hor,
                              predictors_LME,
                              predictors_LME_time,
                              responses_LME,
                              responses_LME_time,
                              individual_id,
                               event_time,
                               event_status){
  if (!(is.data.frame(data_long) ||
        is.list(data_long))) {
    stop("data_long should be a list or data.frame")
  }
  if (is.data.frame(data_long)) {
    data_long <- lapply(x_L, function(x_l) {
      data_long
    })
    names(data_long) <- x_L
  }
  if (is.list(data_long)) {
    if (!setequal(names(data_long), x_L)) {
      stop("Names of elements in data_long should be landmark ages x_L")
    }
  }


  if (!(inherits(predictors_LME,"character"))) {
    stop("predictors_LME should have class character")
  }
  if (!(inherits(predictors_LME_time,"character"))) {
    stop("predictors_LME_time should have class character")
  }
  if (!(inherits(responses_LME,"character"))) {
    stop("responses_LME should have class character")
  }
  if (!(inherits(responses_LME_time,"character"))) {
    stop("responses_LME_time should have class character")
  }
  if (!(inherits(individual_id,"character"))) {
    stop("individual_id should have class character")
  }
  if (!(inherits(event_time,"character"))) {
    stop("event_time should have class character")
  }
  if (!(inherits(event_status,"character"))) {
    stop("event_status should have class character")
  }

  if (!(inherits(x_L,"numeric"))) {
    stop("'x_L' should be numeric")
  }
  if (!(inherits(x_hor,"numeric"))) {
    stop("'x_hor' should be numeric")
  }

  if (!(length(predictors_LME_time) %in% c(length(predictors_LME), 1))) {
    stop("Length of predictors_LME_time should be equal to length of predictors_LME or 1")
  }

  if (length(predictors_LME_time) == 1) {
    predictors_LME_time <- rep(predictors_LME_time, times = length(predictors_LME))
  }
  if (!(length(responses_LME_time) %in% c(length(responses_LME), 1))) {
    stop("Length of responses_LME_time should be equal to length of responses_LME or 1")
  }

  if (length(responses_LME_time) == 1) {
    responses_LME_time <- rep(responses_LME_time, times = length(responses_LME))
  }

  data_long_x_L <- lapply(1:length(x_L), function(i) {

    x_l <- x_L[i]
    x_h <- x_hor[i]

    data_long_x_l <- data_long[[as.character(x_l)]]
    for (col in c(predictors_LME,
                  predictors_LME_time,
                  responses_LME,
                  responses_LME_time,
                  individual_id,
                  event_time,
                  event_status)) {
      if (!(col %in% names(data_long_x_l))) {
        stop(col, " is not a column name in data_long")
      }
      if(any(is.na(data_long_x_l[[col]]))){
        stop(col, " contains NA values")
      }
    }

    data_long_x_l[[individual_id]] <-
      as.factor(data_long_x_l[[individual_id]])

    #Pull out individuals in the risk set
    data_long_x_l_risk_set <-
      return_ids_with_LOCF(
        data_long = data_long_x_l,
        individual_id = individual_id,
        x_L = x_l,
        covariates = predictors_LME,
        covariates_time = predictors_LME_time
      )
    data_long_x_l_risk_set <-
      data_long_x_l_risk_set[data_long_x_l_risk_set[[event_time]] > x_l,]
    n <-
      length(unique(data_long_x_l[[individual_id]])) - length(unique(data_long_x_l_risk_set[[individual_id]]))
    if (n >= 1) {
      warning(
        n,
        " individuals have been removed from the model building as they are not in the risk set at landmark age ",
        x_l
      )
    }
    data_long_x_l <- data_long_x_l_risk_set

    return(data_long_x_l)
  })
  names(data_long_x_L)<-x_L
  data_long_x_L
}

#' Fit a landmarking model using a linear mixed effects (LME) model for the longitudinal data
#'
#' This function is a helper function for `fit_LME_landmark`.
#'
#' @param data_long Data frame containing repeat measurement data and time-to-event data in long format.
#' @template x_L
#' @param standardise_time  Boolean indicating whether to standardise the time variable in the LME model by subtracting the mean
#' and dividing by the standard deviation. See Details section of `fit_LME_longitudinal` for more information.
#' @param cv_name Character string specifying the column name in `data_long` that indicates cross-validation fold
#' @template individual_id
#' @template predictors_LME
#' @template responses_LME
#' @template predictors_LME_time
#' @template responses_LME_time
#' @param random_slope_longitudinal Boolean indicating whether to include a random slope in the LME model. See Details section of `fit_LME_longitudinal` for more information.
#' @param random_slope_survival Boolean indicating whether to include the random slope estimate from the LME model
#' as a covariate in the survival submodel. See Details section of `fit_LME_longitudinal` for more information.
#' @param include_data_after_x_L Boolean indicating whether to include all longitudinal data, including data after the landmark age `x_L`,
#' in the model development dataset. See Details section of `fit_LME_longitudinal` for more information.
#' @param lme_control Object created using `nlme::lmeControl()`, which will be passed to the `control` argument of the `lme` function
#' @return List containing elements:
#' `data_longitudinal`, `model_longitudinal`, `model_LME`, and `model_LME_standardise_time`.
#'
#' `data_longitudinal` has one row for each individual in the risk set at `x_L` and
#' contains the value of the covariates at the landmark time `x_L` of the `predictors_LME` using the LOCF model and
#' `responses_LME` using the LME model.
#'
#' `model_longitudinal` indicates that the LME approach is used.
#'
#' `model_LME` contains the output from
#' the `lme` function from package `nlme`. For a model using cross-validation,
#' `model_LME` contains a list of outputs with each
#' element in the list corresponds to a different cross-validation fold.
#'
#' `model_LME_standardise_time` contains a list of two objects `mean_response_time` and `sd_response_time` if the parameter `standardise_time=TRUE` is used. This
#' is the mean and standard deviation used to normalise times when fitting the LME model.
#'
#' @details For an individual \eqn{i}, the LME model can be written as
#'
#' \deqn{Y_i = X_i \beta + Z_i U_i + \epsilon_i}
#'
#' where
#' * \eqn{Y_i} is the vector of responses at different time points for the individual
#' * \eqn{X_i} is the matrix of predictors for the fixed effects at these time points
#' * \eqn{\beta} is the vector of coefficients for the fixed effects
#' * \eqn{Z_i} is the matrix of predictors for the random effects
#' * \eqn{U_i} is the matrix of coefficients for the random effects
#' * \eqn{\epsilon_i} is the error term, typically from N(0, \eqn{\sigma})
#'
#' By using an LME model to fit repeat measures data, rather than a linear model, we can allow measurements from the same individuals to be
#' more similar than measurements from different individuals. This is done through the random intercept and/or
#' random slope.
#'
#' Extending this model to the case where there are multiple random effects, denoted \eqn{k}, we have
#'
#' \deqn{Y_{ik} = X_{ik} \beta_k + Z_{ik} U_{ik} + \epsilon_{ik}}
#'
#' Typically the random effects are assumed to be from the multivariate normal (MVN) distribution \eqn{MVN(0,\Sigma_u)}
#' and we choose a certain covariance structure for \eqn{\Sigma_u}. The function \code{fit_LME_landmark} uses this distribution with
#' unstructured covariance for the random effects when fitting the LME model (i.e. no constraints are imposed on the values).
#'
#' To fit the LME model the function \code{lme} from the package \code{nlme} is used.
#' The random intercept is always included in the LME model.
#' Additionally, the random slope can be included in the LME model using the parameter `random_slope_longitudinal=TRUE`.
#'
#' It is important to distinguish between the validation set and the development set for fitting the LME model in this function.
#' The development dataset either includes all the repeat measurements (including those after the landmark age \code{x_L}), or only the repeat measurements
#' recorded up to and including the landmark age \code{x_L}. This is controlled using the parameter `include_data_after_x_L`.
#' The validation set only includes the repeat measurements recorded up until and including the landmark age \code{x_L},
#' i.e. it does not include future data in its predictions.
#'
#' Using the fitted model, the values of the best linear unbiased predictions (BLUPs)
#' at the landmark age \code{x_L} are calculated. These BLUPs are the predictions of the values of the \code{responses_LME}
#' the landmark age \code{x_L}. The values of the predictors in this prediction are the LOCF values of the \code{predictors_LME}
#' at the landmark age \code{x_L}. In the function `fit_LME_landmark`, these predictions are used as covariates in the survival
#' model along with the LOCF values of  \code{predictors_LME}. Additionally, the estimated value of the random slope can
#' be included as predictors in the survival model using the parameter `random_slope_survival=TRUE`.
#'
#' There is an important consideration about fitting the linear mixed effects model. As the variable \code{responses_LME_time}
#' gets further from 0, the random effects coefficients get closer to 0. This causes computational issues
#' as the elements in the covariance matrix of the random effects, \eqn{\Sigma_u}, are constrained to
#' be greater than 0. Using parameter \code{standard_time=TRUE} can prevent this issue by standardising the
#' time variables to ensure that the \code{responses_LME_time} values are not too close to 0.
#'
#'
#' @export
fit_LME_longitudinal <- function(data_long,
                                 x_L,
                                 predictors_LME,
                                 responses_LME,
                                 predictors_LME_time,
                                 responses_LME_time,
                                 standardise_time = FALSE,
                                 random_slope_longitudinal = TRUE,
                                 random_slope_survival = TRUE,
                                 include_data_after_x_L=TRUE,
                                 cv_name = NA,
                                 individual_id,
                                 lme_control = nlme::lmeControl()) {
  call <- match.call()
  if (!(is.data.frame(data_long))) {
    stop("data_long should be a dataframe")
  }
  if (!(is.numeric(x_L))) {
    stop("'x_L' should be numeric")
  }

  for (col in c(
    predictors_LME,
    responses_LME,
    predictors_LME_time,
    responses_LME_time,
    individual_id
  )) {
    if (!(col %in% names(data_long))) {
      stop(col, " is not a column name in data_long")
    }
  }

  if (!is.na(cv_name)) {
    if (!(cv_name %in% names(data_long))) {
      stop(cv_name, " is not a column name in data_long")
    }
    if (any(is.na(data_long[[cv_name]]))) {
      stop("The column ", cv_name, " contains NA values")
    }
  }

  if (is.na(cv_name)) {
    data_long[["cross_validation_number"]] <-
      1
    cv_name <- "cross_validation_number"
  }

  if (!(length(predictors_LME_time) %in% c(length(predictors_LME), 1))) {
    stop("Length of predictors_LME_time should be equal to length of predictors_LME or 1")
  }

  if (length(predictors_LME_time) == 1) {
    predictors_LME_time <-
      rep(predictors_LME_time, times = length(predictors_LME))
  }

  if (!(length(responses_LME_time) %in% c(length(responses_LME), 1))) {
    stop("Length of responses_LME_time should be equal to length of responses_LME or 1")
  }

  if (length(responses_LME_time) == 1) {
    responses_LME_time <-
      rep(responses_LME_time, times = length(responses_LME))
  }

  if (dim(
    return_ids_with_LOCF(
      data_long = data_long,
      individual_id = individual_id,
      x_L = x_L,
      covariates = predictors_LME,
      covariates_time = predictors_LME_time
    )
  )[1] != dim(data_long)[1]) {
    stop(
      "data_long contains individuals that do not have a LOCF for all predictors_LME. Use function return_ids_with_LOCF to remove these individuals from the dataset data_long."
    )
  }

  data_long[[individual_id]] <-
    as.factor(data_long[[individual_id]])

  data_LOCF <- data_long
  data_LME <- data_long

  #Pick out LOCF for each variable
  LOCF_values_by_variable <-
    lapply(1:length(c(predictors_LME, responses_LME)), function(x) {
      return_LOCF_by_variable(
        data_long = data_LOCF,
        i = x,
        covariates = c(predictors_LME, responses_LME),
        covariates_time = c(predictors_LME_time, responses_LME_time),
        individual_id = individual_id,
        x_L =
          x_L
      )
    })
  data_LOCF <- Reduce(merge, LOCF_values_by_variable)

  data_LOCF <-
    dplyr::left_join(data_LOCF, unique(data_long[c(individual_id, cv_name)]), by =
                       individual_id)

  #Create validation and development dataset
  response_type <-
    Reduce(c, lapply(responses_LME, function(i) {
      rep(i, dim(data_LME)[1])
    }))
  response <-
    as.numeric(Reduce(c, lapply(1:length(responses_LME), function(i) {
      data_LME[, responses_LME[i]]
    })))
  response_time <-
    as.numeric(Reduce(c, lapply(1:length(responses_LME), function(i) {
      data_LME[, responses_LME_time[i]]
    })))
  data_predictors_LME <-
    do.call("rbind", replicate(
      n = length(responses_LME),
      data_LME[, c(individual_id, predictors_LME, cv_name)],
      simplify = FALSE
    ))
  data_LME <-
    data.frame(data_predictors_LME, response_type, response, response_time)

  #Standardise response time
  if (standardise_time == TRUE) {
    mean_response_time <- mean(data_LME$response_time, na.rm = TRUE)
    sd_response_time <- stats::sd(data_LME$response_time, na.rm = TRUE)
  } else{
    mean_response_time <- 0
    sd_response_time <- 1
  }
  data_LME$response_time <-
    (data_LME$response_time - mean_response_time) / sd_response_time
  x_L <- (x_L - mean_response_time) / sd_response_time
  standardise_time <-
    list(mean_response_time = mean_response_time,
         sd_response_time = sd_response_time)

  data_LME_model_val <- data_LME[data_LME$response_time <= x_L, ] #validation set
  data_LME_model_dev <- data_LME[!is.na(data_LME$response), ] #development set

  if (include_data_after_x_L==FALSE){data_LME_model_dev <- data_LME_model_dev[data_LME_model_dev$response_time <= x_L, ]}

  if (length(responses_LME) == 1){
    formula_weights <- NULL
    if (random_slope_longitudinal == FALSE) {
      formula_random <- as.formula(paste0(" ~ 1 |", individual_id))
    }else{
      formula_random <-
        as.formula(paste0(" ~ 1 + response_time |", individual_id))
    }
    formula_fixed <-
      as.formula(paste0(c(
        paste0("response ~ 1 "), c("response_time", predictors_LME)
      ), collapse = "+"))
  }
  if (length(responses_LME) > 1) {
    formula_weights <- nlme::varIdent(form = ~ 1 | "response_type")
    if (random_slope_longitudinal == FALSE) {
      formula_random <-
        as.formula(paste0(" ~ -1 + response_type | ", individual_id))
    }
    else{
      formula_random <-
        as.formula(paste0(
          " ~ -1 + response_type + response_type:response_time | ",
          individual_id
        ))
    }
    formula_fixed <- as.formula(paste0(c(
      paste0(c(
        paste0("response~-1+ response_type"),
        c("response_time", predictors_LME)
      ), collapse = "+"),
      paste0(paste0(paste0(
        c("response_time", predictors_LME), ":response_type"
      )), collapse = "+")
    ), collapse = "+"))
  }

  cv_numbers <- unique(data_LME_model_dev[[cv_name]])
  model_LME <- lapply(cv_numbers, function(cv_number) {
    if (length(cv_numbers) > 1) {
      data_dev_cv <-
        data_LME_model_dev[data_LME_model_dev[[cv_name]] != cv_number, ]
    }
    if (length(cv_numbers) == 1) {
      data_dev_cv <- data_LME_model_dev
    }
    model_LME_cv <- nlme::lme(
      fixed = formula_fixed,
      random = formula_random,
      data = data_dev_cv,
      weights = formula_weights,
      control = lme_control
    )
    model_LME_cv$call$fixed <- formula_fixed
    model_LME_cv
  })
  names(model_LME) <- cv_numbers

  response_type <-
    Reduce(c, lapply(responses_LME, function(i) {
      rep(i, dim(data_LOCF)[1])
    }))
  response <- as.numeric(rep(NA, length(response_type)))
  response_time <- as.numeric(rep(x_L, length(response_type)))
  data_predictors_LME <-
    do.call("rbind", replicate(
      n = length(responses_LME),
      data_LOCF[, c(individual_id, predictors_LME, cv_name)],
      simplify = FALSE
    ))
  data_LOCF_model_val <-
    data.frame(data_predictors_LME,
               response_type,
               response,
               response_time,
               predict = 1)
  data_LME_model_val$predict <- 0
  data_LME_model_val <-
    dplyr::bind_rows(data_LOCF_model_val, data_LME_model_val)

  data_LME <-
    lapply(cv_numbers, function(cv_number) {
      data_LME_model_val_cv <-
        data_LME_model_val[which(data_LME_model_val[[cv_name]] == cv_number),]
      data_LME_model_val_cv <- droplevels(data_LME_model_val_cv)
      model_LME_cv <- model_LME[[as.character(cv_number)]]
      response_predictions <-
        which(data_LME_model_val_cv$predict == 1)

      mixoutsamp_LME_model_val_cv <-
        mixoutsamp(model = model_LME_cv,
                   newdata = data_LME_model_val_cv)

      data_LME_model_val_cv <-
        mixoutsamp_LME_model_val_cv$preddata[response_predictions,][, c(individual_id,
                                                                        predictors_LME,
                                                                        cv_name,
                                                                        "response_type",
                                                                        "fitted")]
      data_LME_model_val_cv <-
        stats::reshape(
          data_LME_model_val_cv,
          timevar = "response_type",
          idvar = c(individual_id, predictors_LME, cv_name),
          direction = "wide"
        )

      data_LME_model_val_cv<-dplyr::rename_with(data_LME_model_val_cv,~responses_LME[which(paste0("fitted.",responses_LME)==.x)],.cols=paste0("fitted.",responses_LME))

      if (random_slope_survival == TRUE) {
        if (length(responses_LME)==1){
          slopes_df<-mixoutsamp_LME_model_val_cv$random[,c("id","reffresponse_time")]
          slopes_df["reffresponse_time"]<-slopes_df["reffresponse_time"]+model_LME_cv$coefficients$fixed["response_time"]

        }
        if (length(responses_LME)>1){
          slopes_df<-mixoutsamp_LME_model_val_cv$random[,c("id",paste0("reffresponse_type",responses_LME,":response_time"))]
          responses_LME_dummy<-paste0("response_type",responses_LME,":response_time")
          responses_LME_dummy[1]<-"response_time"
          for (i in 1:length(responses_LME)){
            slopes_df[,paste0("reffresponse_type",responses_LME[i],":response_time")]<-
               slopes_df[,paste0("reffresponse_type",responses_LME[i],":response_time")]+model_LME_cv$coefficients$fixed[responses_LME_dummy[1]]
            if(i!=1){slopes_df[,paste0("reffresponse_type",responses_LME[i],":response_time")]<-
              slopes_df[,paste0("reffresponse_type",responses_LME[i],":response_time")]+model_LME_cv$coefficients$fixed[responses_LME_dummy[i]]}
          }
        }
      names(slopes_df)<-c("id",paste0(responses_LME,"_slope"))
      data_LME_model_val_cv <- dplyr::left_join(data_LME_model_val_cv,
                                                slopes_df,
                                                by = individual_id)
      }

      data_LME_model_val_cv[[as.character(cv_name)]] <- cv_number
      data_LME_model_val_cv
    })
  data_LME <- do.call("rbind", data_LME)

  data_LME <-
    data_LME[match(unique(data_long[[individual_id]][data_long[[individual_id]] %in% data_LME[[individual_id]]]), data_LME[[individual_id]]), ]
  data_LME <-
    data_LME[, order(match(names(data_LME), names(data_long)))]
  rownames(data_LME) <- NULL

  if (length(cv_numbers) == 1) {
    model_LME <- model_LME[[1]]
  }
  if (length(unique(data_long[[cv_name]])) == 1) {
    data_LME[[cv_name]] <- NULL
  }

  return(
    list(
      data_longitudinal = data_LME,
      model_longitudinal = "LME",
      call = call,
      model_LME = model_LME,
      model_LME_standardise_time = standardise_time
    )
  )
}


#' Fit a landmarking model using a linear mixed effects (LME) model for the longitudinal submodel
#'
#' This function performs the two-stage landmarking analysis.
#'
#' @param data_long Data frame or list of data frames each corresponding to a landmark age `x_L` (each element of the list must be named the value of `x_L` it corresponds to).
#' Each data frame contains repeat measurements data and time-to-event data in long format.
#' @template x_L
#' @template x_hor
#' @param standardise_time Boolean indicating whether to standardise the time variable in the LME model by subtracting the mean
#' and dividing by the standard deviation. See Details section of `fit_LME_longitudinal` for more information.
#' @template event_status
#' @template event_time
#' @template k
#' @template cross_validation_df
#' @template individual_id
#' @template predictors_LME
#' @template responses_LME
#' @template predictors_LME_time
#' @template responses_LME_time
#' @param include_data_after_x_L Boolean indicating whether to include all longitudinal data, including data after the landmark age `x_L`,
#' in the model development dataset. See Details section of `fit_LME_longitudinal` for more information.
#' @param random_slope_longitudinal Boolean indicating whether to include a random slope in the LME model. See Details section of `fit_LME_longitudinal` for more information.
#' @param random_slope_survival Boolean indicating whether to include the random slope estimate from the LME model.
#' See Details section of `fit_LME_longitudinal` for more information.
#' as a covariate in the survival submodel. See Details section of `fit_LME_longitudinal` for more information.
#' @param lme_control Object created using `nlme::lmeControl()`, which will be passed to the `control` argument of the `lme`
#' function
#' @template b
#' @template survival_submodel
#' @return List containing containing information about the landmark model at each of the landmark times.
#' Each element of this list is named the corresponding landmark time, and is itself a list containing elements:
#' `data`, `model_longitudinal`, `model_LME`, `model_LME_standardise_time`, `model_survival`, and `prediction_error`.
#'
#' `data` has one row for each individual in the risk set at `x_L` and
#' contains the value of the `predictors_LME` using the LOCF approach and predicted values of the
#' `responses_LME` using the LME model at the landmark time `x_L`. It also includes the predicted
#' probability that the event of interest has occurred by time \code{x_hor}, labelled as \code{"event_prediction"}.
#' There is one row for each individual.
#'
#' `model_longitudinal` indicates that the longitudinal approach is LME.
#'
#' `model_LME` contains the output from
#' the `lme` function from package `nlme`. For a model using cross-validation,
#' `model_LME` contains a list of outputs with each
#' element in the list corresponds to a different cross-validation fold.
#'
#' `model_LME_standardise_time` contains a list of two objects `mean_response_time` and `sd_response_time` if the parameter `standardise_time=TRUE` is used. This
#' is the mean and standard deviation use to normalise times when fitting the LME model.
#'
#' `model_survival` contains the outputs from
#' the survival submodel functions, including the estimated parameters of the model. For a model using cross-validation,
#' `model_survival` will contain a list of outputs with each
#' element in the list corresponding to a different cross-validation fold.
#'
#' `prediction_error` contains a list indicating the c-index and Brier score at time `x_hor` and their standard errors if parameter `b` is used.
#' @details Firstly, this function selects the individuals in the risk set at the landmark time \code{x_L}.
#' Specifically, the individuals in the risk set are those that have entered the study before the landmark time \code{x_L}
#' (there is at least one observation for each of the \code{predictors_LME} and \code{responses_LME} on or before \code{x_L}) and
#' exited the study after the landmark age (\code{event_time}
#' is greater than \code{x_L}).
#'
#' Secondly, if the option to use cross validation
#' is selected (using either parameter `k` or `cross_validation_df`), then an extra column `cross_validation_number` is added with the
#' cross-validation folds. If parameter `k` is used, then the function `add_cv_number`
#' randomly assigns these folds. For more details on this function see `?add_cv_number`.
#' If the parameter `cross_validation_df` is used, then the folds specified in this data frame are added.
#' If cross-validation is not selected then the landmark model is
#' fit to the entire group of individuals in the risk set (this is both the training and test dataset).
#'
#' Thirdly, the landmark model is then fit to each of the training datasets. There are two parts to fitting the landmark model: using the longitudinal data and using the survival data.
#' Using the longitudinal data is the first stage and is performed using `fit_LME_longitudinal`. See `?fit_LME_longitudinal` more for information about this function.
#' Using the survival data is the second stage and is performed using `fit_survival_model`. This function censors the
#' individuals at the time horizon `x_L` and fits the survival model. See `?fit_survival_model` more for information about this function.
#'
#' Fourthly, the performance of the model is then assessed on the set of predictions
#' from the entire set of individuals in the risk set by calculating Brier score and C-index.
#' This is performed using `get_model_assessment`. See `?get_model_assessment` more for information about this function.
#'
#' @author Isobel Barrott \email{isobel.barrott@@gmail.com}
#' @examples \donttest{
#' library(Landmarking)
#' data(data_repeat_outcomes)
#' data_model_landmark_LME <-
#'   fit_LME_landmark(
#'     data_long = data_repeat_outcomes,
#'     x_L = c(60, 61),
#'     x_hor = c(65, 66),
#'     k = 10,
#'     predictors_LME = c("ethnicity", "smoking", "diabetes"),
#'     predictors_LME_time = "response_time_sbp_stnd",
#'     responses_LME = c("sbp_stnd", "tchdl_stnd"),
#'     responses_LME_time = c("response_time_sbp_stnd", "response_time_tchdl_stnd"),
#'     individual_id = "id",
#'     standardise_time = TRUE,
#'     lme_control = nlme::lmeControl(maxIter = 100, msMaxIter = 100),
#'     event_time = "event_time",
#'     event_status = "event_status",
#'     survival_submodel = "cause_specific"
#'   )
#' }
#' @importFrom stats as.formula
#' @importFrom prodlim Hist
#' @export


fit_LME_landmark <- function(data_long,
                             x_L,
                             x_hor,
                             predictors_LME,
                             responses_LME,
                             predictors_LME_time,
                             responses_LME_time,
                             random_slope_longitudinal=TRUE,
                             random_slope_survival=TRUE,
                             include_data_after_x_L=TRUE,
                             individual_id,
                             k,
                             cross_validation_df,
                             standardise_time = FALSE,
                             lme_control = nlme::lmeControl(),
                             event_time,
                             event_status,
                             survival_submodel = c("standard_cox", "cause_specific", "fine_gray"),
                             b) {
  call <- match.call()

  survival_submodel <- match.arg(survival_submodel)


  #Checks
  if (missing(k)) {
    k_add <- FALSE
  }
  else{
    k_add <- TRUE
    if (!(is.numeric(k))) {
      stop("k should be numeric")
    }
  }

  if (missing(cross_validation_df)) {
    cross_validation_df_add <- FALSE
  }else{
    cross_validation_df_add <- TRUE
    if (inherits(cross_validation_df, "list")) {
      if (!all(x_L %in% names(cross_validation_df))) {
        stop(
          "The names of elements in cross_validation_df list should be the landmark times in x_L"
        )
      }
      if (any(Reduce("c", lapply(cross_validation_df, function(x) {
        any(duplicated(dplyr::distinct(x[, c(individual_id, "cross_validation_number")])[, individual_id]))
      })))) {
        stop("Cross validation folds should be the same for the same individual")
      }
    }
    else if (inherits(cross_validation_df, "data.frame")) {
      if (any(duplicated(dplyr::distinct(cross_validation_df[, c(individual_id, "cross_validation_number")])[, individual_id]))) {
        stop("Cross validation folds should be the same for the same individual")
      }
      cross_validation_df<-list(cross_validation_df)
      names(cross_validation_df)<-x_L
    }
    else{
      stop("cross_validation_df should be either a data frame or a list")
    }
  }
  if (k_add == TRUE &&
      cross_validation_df_add == TRUE) {
    stop("Either use parameter k or cross_validation_df but not both")
  }
  if (k_add == FALSE &&
      cross_validation_df_add == FALSE) {
    cv_name <- NA
  } else{
    cv_name <- "cross_validation_number"
  }

  if (!(length(predictors_LME_time) %in% c(length(predictors_LME), 1))) {
    stop("Length of predictors_LME_time should be equal to length of predictors_LME or 1")
  }
  if (length(predictors_LME_time) == 1) {
    predictors_LME_time <-
      rep(predictors_LME_time, times = length(predictors_LME))
  }
  if (!(length(responses_LME_time) %in% c(length(responses_LME), 1))) {
    stop("Length of responses_LME_time should be equal to length of responses_LME or 1")
  }
  if (length(responses_LME_time) == 1) {
    responses_LME_time <-
      rep(responses_LME_time, times = length(responses_LME))
  }

  if (length(x_L) != length(x_hor)) {
    stop("Length of x_L should be the same as length of x_hor")
  }

  if (!(is.data.frame(data_long) ||
        is.list(data_long))) {
    stop("data_long should be a list or data.frame")
  }
  if (is.data.frame(data_long)) {
    data_long <- lapply(x_L, function(x_l) {
      data_long
    })
    names(data_long) <- x_L
  }
  if (is.list(data_long)) {
    if (!setequal(names(data_long), x_L)) {
      stop("Names of elements in data_long should be landmark ages x_L")
    }
  }

  if (missing(b)) {
    b <- NA
  }
  #Find risk set
  data_long_x_L<-find_LME_risk_set(data_long=data_long,
                                    x_L=x_L,
                                    x_hor=x_hor,
                                   predictors_LME = predictors_LME,
                                   predictors_LME_time=predictors_LME_time,
                                   responses_LME = responses_LME,
                                   responses_LME_time=responses_LME_time,
                                   individual_id=individual_id,
                                    event_time=event_time,
                                    event_status=event_status)


  #Add cross-validation folds
  if (cross_validation_df_add == TRUE) {
    for (x_l in x_L){
      data_long_x_L[[as.character(x_l)]]<-
        dplyr::left_join(data_long_x_L[[as.character(x_l)]], cross_validation_df[[as.character(x_l)]][,c(individual_id,"cross_validation_number")],by=individual_id)
    }
    if(any(is.na(data_long_x_L[[as.character(x_l)]][,"cross_validation_number"]))){stop("Cross validation number not defined for all in individual_id")}
  }

  if (k_add == TRUE) {
    data_long_x_L_cv <-
      add_cv_number(
        data_long = Reduce("rbind", data_long_x_L),
        individual_id = individual_id,
        k = k
      )
    data_long_x_L <-
      lapply(data_long_x_L, function(x) {
        dplyr::left_join(x, dplyr::distinct(data_long_x_L_cv[, c(individual_id, "cross_validation_number")]), by =
                           individual_id)
      }
    )
  }

  out <- lapply(1:length(x_L), function(i) {
    x_l <- x_L[i]
    x_h <- x_hor[i]

    data_long <- data_long_x_L[[as.character(x_l)]]
    message("Fitting longitudinal submodel, landmark age ", x_l)

    data_model_longitudinal <-
      fit_LME_longitudinal(
        data_long = data_long,
        x_L = x_l,
        predictors_LME =
          predictors_LME,
        responses_LME =
          responses_LME,
        predictors_LME_time =
          predictors_LME_time,
        responses_LME_time =
          responses_LME_time,
        random_slope_longitudinal = random_slope_longitudinal,
        random_slope_survival =
          random_slope_survival,
        include_data_after_x_L=include_data_after_x_L,
        standardise_time =
          standardise_time,
        cv_name = cv_name,
        individual_id =
          individual_id,
        lme_control = lme_control
      )
    message("Complete, landmark age ", x_l)

    data_events <-
      dplyr::distinct(data_long[, c(individual_id, event_status, event_time)])
    data_longitudinal <-
      dplyr::left_join(data_model_longitudinal$data_longitudinal,
                       data_events,
                       by = individual_id)

    message("Fitting survival submodel, landmark age ", x_l)

    if (random_slope_survival) {
      responses_LME <- c(responses_LME, paste0(responses_LME, "_slope"))

    }
    data_model_survival <- fit_survival_model(
      data = data_longitudinal,
      individual_id = individual_id,
      cv_name = cv_name,
      covariates = c(predictors_LME, responses_LME),
      event_time = event_time,
      event_status = event_status,
      survival_submodel = survival_submodel,
      x_hor = x_h
    )

    data_events <-
      dplyr::left_join(data_events,
                       data_model_survival$data_survival[,c(individual_id,"event_prediction")],
                       by = individual_id)

    message("Complete, landmark age ", x_l)

    prediction_error <-
      get_model_assessment(
        data = data_model_survival$data_survival,
        individual_id = individual_id,
        event_prediction = "event_prediction",
        event_status = event_status,
        event_time = event_time,
        x_hor =  x_h,
        b = b
      )

    list(
      data = data_model_survival$data_survival,
      model_longitudinal = data_model_longitudinal$model_longitudinal,
      model_LME = data_model_longitudinal$model_LME,
      model_LME_standardise_time = data_model_longitudinal$model_LME_standardise_time,
      model_survival = data_model_survival$model_survival,
      prediction_error = prediction_error,
      call = call
    )
  })
  names(out) <- x_L
  class(out) <- "landmark"
  out
}

#' Compute C-index and Brier score
#'
#' Performs model assessment by computing the C-index and Brier score at time `x_hor`. There is the option
#' to calculate their standard errors using bootstraping.
#'
#' @param data Data frame containing survival outcomes and the event predictions from the model,
#' there should be one row for each individual
#' @param event_status Character string specifying the column name in `data` which contains the event status (where 0=censoring, 1=event of interest, if there are competing events these are labelled 2 or above). Events at time `x_hor` should be labelled censored.
#' @param event_time Character string specifying the column name in `data` which contains the event time.
#' @template x_hor
#' @param individual_id  Character string specifying the column name in `data` which contains the individual identifiers
#' @template event_prediction
#' @template b
#' @return List containing C-index, Brier score and their standard errors
#' @details There are two factors in assessing the performance of a prediction model; its
#' discrimination and its calibration. The c-index is a commonly used metric which assesses
#' discrimination, this refers to the ability of the model to separate individuals into
#' those that will have an event and those that will not. The c-index at a horizon time `x_hor`
#' looks at the pairs of individuals where one individual has the event at a time T and the other has not had the event at time T.
#' It is calculated as the proportion of these pairs where their relative risk prediction agrees with the
#' actual outcomes for the two individuals. This is extended to the competing risks case
#' by comparing individuals where one had the event of interest at time T and the other individual either
#' did not experience the event before this time T or experienced a competing event.
#'
#' The Brier score gives an indication of the calibration of a model (and its discrimination to an extent), this refers to the agreement between the risk prediction and
#' the outcome. The Brier score is calculated as the average mean squared error of the predicted risk and the event outcome (where
#' an event is 1 and not experiencing the event is 0). This is extended to the competing risks case by including the competing risk events as
#' not experiencing the event.
#'
#' For both the c-index and Brier score calculations, inverse probability censoring weighting (IPCW) is used to create weights
#' which account for the occurrence of censoring. The censoring model assumes for this function is the Kaplan Meier model, i.e. censoring occurs
#' independently of covariates.
#'
#' The c-index is calculated using the `cindex` function in package `pec`. The Brier score is calculated using
#' `pec` function in package `pec`.
#'
#'
#' @author Isobel Barrott \email{isobel.barrott@@gmail.com}
#' @examples \dontrun{
#' library(Landmarking)
#' data(data_repeat_outcomes)
#' data_model_landmark_LOCF <-
#'   fit_LOCF_landmark(
#'     data_long = data_repeat_outcomes,
#'     x_L = c(60, 61),
#'     x_hor = c(65, 66),
#'     covariates =
#'       c("ethnicity", "smoking", "diabetes", "sbp_stnd", "tchdl_stnd"),
#'     covariates_time =
#'       c(rep("response_time_sbp_stnd", 4), "response_time_tchdl_stnd"),
#'     k = 10,
#'     individual_id = "id",
#'     event_time = "event_time",
#'     event_status = "event_status",
#'     survival_submodel = "cause_specific"
#'   )
#' get_model_assessment(data = data_model_landmark_LOCF[["60"]]$data,
#'   individual_id = "id",
#'   event_prediction = "event_prediction",
#'   event_status = "event_status",
#'   event_time = "event_time",
#'   x_hor = 65,
#'   b = 100)}
#' @export


get_model_assessment <-
  function(data,
           individual_id,
           event_prediction,
           event_status,
           event_time,
           x_hor,
           b) {
    if (!(inherits(data,"data.frame"))) {
      stop("data should be a data frame")
    }
    if (!(inherits(individual_id,"character"))) {
      stop("individual_id should have class character")
    }
    if (!(inherits(event_prediction,"character"))) {
      stop("event_prediction should have class character")
    }
    if (!(inherits(event_status,"character"))) {
      stop("event_status should have class character")
    }
    if (!(inherits(event_time,"character"))) {
      stop("event_time should have class character")
    }
    for (col in c(individual_id,
                  event_prediction,
                  event_status,
                  event_time)) {
      if (!(col %in% names(data))) {
        stop(col, " is not a column name in data")
      }
      if(any(is.na(data[[col]]))){
        stop(col, " contains NA values")
      }
    }

    if (!(inherits(x_hor,"numeric"))) {
      stop("x_hor should be numeric")
    }
    if (!is.na(b)) {
      if (!(inherits(b,"numeric"))) {
        stop("b should be numeric")
      }
      standard_error <- TRUE
    }
    else{
      standard_error <- FALSE
    }
    if (length(unique(data[[individual_id]])) != dim(data)[1]) {
      stop("There should be one row for each individual in data")
    }

    n <- dim(data)[1]
    Hist <- prodlim::Hist
    Surv <- survival::Surv

    data[["event_time"]] <- data[[event_time]]
    data[["event_status"]] <- data[[event_status]]
    data[["event_prediction"]] <- data[[event_prediction]]

    if (setequal(data[[event_status]],0:1)){ # survival probabilities are needed for the non-competing risks model
      data[["event_prediction"]] <- 1 - data[["event_prediction"]]
    }

    c_index <-
      pec::cindex(
        object = matrix(data[["event_prediction"]]),
        formula = Hist(event_time, event_status) ~ 1,
        cause = 1,
        data = data,
        cens.model = "marginal",
        splitMethod = "none",
        B = 0,
        exact = FALSE,
        verbose = FALSE,
        eval.times = x_hor
      )$AppCindex$matrix

    if (standard_error == TRUE) {
      rho_hat_star <-
        unlist(lapply(1:b, function(i) {
          c_index_bootstrap(
            n = n,
            data = data,
            event_prediction =
              event_prediction,
            event_time =
              event_time,
            event_status =
              event_status,
            x_hor =
              x_hor
          )
        }))
      c_index_error <-
        sqrt((1 / (b - 1)) * (sum((
          rho_hat_star - mean(rho_hat_star)
        ) ^ 2)))
    }

    brier_score <-
      pec::pec(
        object = matrix(cbind(0, data[["event_prediction"]]), ncol =
                          2),
        formula = Hist(event_time, event_status) ~ 1,
        cause = 1,
        data = data,
        cens.model = "marginal",
        splitMethod = "none",
        B = 0,
        exact = FALSE,
        verbose = FALSE,
        reference = FALSE,
        times = x_hor
      )$AppErr$matrix[2]

    if (standard_error == TRUE) {
      rho_hat_star <-
        unlist(lapply(1:b, function(i) {
          brier_bootstrap(
            n = n,
            data = data,
            event_prediction =
              event_prediction,
            event_time =
              event_time,
            event_status =
              event_status,
            x_hor =
              x_hor
          )
        }))
      brier_score_error <-
        sqrt((1 / (b - 1)) * (sum((
          rho_hat_star - mean(rho_hat_star)
        ) ^ 2)))


    }


    if (standard_error == FALSE) {
      c_index_error <- NA
      brier_score_error <- NA
    }

    return(
      list(
        c_index = c_index,
        c_index_standard_error = c_index_error,
        brier_score = brier_score,
        brier_score_standard_error = brier_score_error
      )
    )
  }

c_index_bootstrap <-
  function(n,
           data,
           event_prediction,
           event_time,
           event_status,
           x_hor) {
    bootstrap_sample <- sample(n, replace = TRUE)
    Hist <- prodlim::Hist
    Surv <- survival::Surv

    data[["event_time"]] <- data[[event_time]]
    data[["event_status"]] <- data[[event_status]]
    data[["event_prediction"]] <- data[[event_prediction]]

    pec::cindex(
      object = matrix(data[["event_prediction"]][bootstrap_sample]),
      formula = Hist(event_time, event_status) ~ 1,
      cause = 1,
      data = data[bootstrap_sample, ],
      cens.model = "marginal",
      splitMethod = "none",
      B = 0,
      exact = FALSE,
      verbose = FALSE,
      eval.times = x_hor
    )$AppCindex$matrix
  }

brier_bootstrap <-
  function(n,
           data,
           event_prediction,
           event_time,
           event_status,
           x_hor) {
    bootstrap_sample <- sample(n, replace = TRUE)

    data[["event_time"]] <- data[[event_time]]
    data[["event_status"]] <- data[[event_status]]
    data[["event_prediction"]] <- data[[event_prediction]]

    pec::pec(
      object = matrix(cbind(0, data[["event_prediction"]][bootstrap_sample]), ncol =
                        2),
      formula = Hist(event_time, event_status) ~ 1,
      cause = 1,
      data = data[bootstrap_sample, ],
      cens.model = "marginal",
      splitMethod = "none",
      B = 0,
      exact = FALSE,
      verbose = FALSE,
      reference = FALSE,
      times = x_hor
    )$AppErr$matrix[2]
  }

---
title: "Introduction to Landmark Models and the R package Landmarking"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to Landmark Models and the R package Landmarking}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


## What is the landmark model used for?

Landmark models are used to **dynamically predict** the risk of an event for an individual. This means that an individual has a personalised risk prediction which is updated as new information is collected about them. Landmark models are particularly applicable in a medical setting where electronic health records (EHRs), such as GP records, are continually being updated with new data.

## What is the landmark model?

The key idea behind landmark models is the concept of predefined **landmark times**. A landmark time is a time point (i.e. "landmark") that we wish to make a risk prediction at. A model is built at the landmark time and this model is developed only using the data of individuals at risk (i.e. not censored or having experienced an event) at the landmark time. 

>For example, suppose that we want to predict the 5 year risk of a cardiovascular disease (CVD) event for a 45 year old patient. Then we would create a model for landmark time 45 years old using all the individuals that are in the risk set at 45 years old, and use this model to predict the risk of an event occurring for our new individual.

 As we want to predict the risk of an event for an individual at a selection of time points, we define multiple landmark times and develop a model for each of these. 
 
>Continuing the example and supposing we wish to be able to predict the risk of CVD disease for any age between 45 and 50 years old, we define six landmark ages (45, ... , 50 years old) and fit six models for each of these. This is demonstrated in Figure 1.

```{r figurename, echo=FALSE, fig.cap="*Figure 1: Illustration of a landmark model with six landmark times (ages 45 to 50 years). The blue solid line represents the time interval of longitudinal (repeat measures) data that feeds into the first stage of the two-stage landmark model. The red dashed line represents the time interval of the time-to-event data to be inputted into the survival model which is the second stage of the landmark model. Events after the time horizon need to be censored within this framework.*"}
knitr::include_graphics("../vignettes/landmarking_models.png")
```

To form a model at each landmark age, landmark models use a **two-stage model** that is comprised of a extracting information from the **longitudinal data** which then feeds into **survival model**. The typical form of this is the last observation carried forward (LOCF) method and the Cox proportional hazards model respectively, although there are extensions and these are discussed in the `Features of the R package Landmarking' section. Figure 1 shows the time intervals of the longitudinal (repeat measures) data and time-to-event data that is included when using the model, which is relative to the landmark age and time horizon. It should be noted that all the longitudinal (repeat measures) data may be used, including those passed the landmark age, in the first stage to develop a model. The survival model is fit only for the time-to-event data between the landmark age and the time horizon with all individuals censored at the time horizon. For a Cox proportional hazard survival  model, this has the advantage that proportional hazards is only assumed for the short period between the landmark time and the time horizon rather than the entire period up until the end of study.

## What are the advantages of the landmark model?

The other main statistical framework for this dynamic prediction problem is joint modelling. In comparison to joint modelling, the landmarking model is less computationally intensive and makes a less strong assumption about proportional hazards. Specifically this assumption only applies to the period between the landmark time and time horizon when using the landmark model.

## Features of the R package Landmarking

The package **Landmarking** offers benefits over existing packages *dynpred* and *landpred* for landmark model analysis. In particular, three features of the package are outlined below.

### 1. Linear mixed effects modelling 

The LME model is an alternative to using the LOCF method for the longitudinal data. LME modelling means that the entire history of a risk factor is used to evaluate predicted risk, as opposed to the LOCF model where only one (the most recent) measurement is used. Moreover the LME allows a patient to have **missing values** for the variables in the response.

Figure 2 illustrates the use of these two types of models to predict systolic blood pressure at the landmark time. The LOCF model simply uses the most recently observed value of a risk factor. On the other hand, the LME model uses the entire history of the values of the risk factor from the individual, in combination with the trend in the risk factor from individuals used to develop the model. Using the Landmarking package, it is possible include a covariance structure in the random effects meaning that other risk factors can inform predictions. This improves the accuracy of the estimated value of the risk factor at the landmark time value compared to the LOCF model.

```{r figurename2, echo=FALSE, fig.cap="*Figure 2: Comparison of the LOCF and LME model to predict systolic blood pressure at a landmark age for an individual with 8 repeat measures.*"}
knitr::include_graphics("../vignettes/landmarking_lme_model.png")
```

### 2. Competing risks modelling

The Landmarking R package has the ability to fit competing risks models for the survival model. There are two types of competing risks model that can be fit using this package, the Fine Gray model and the cause-specific model, using the parameter *survival_submodel* in the function *fit_LOCF_landmark* or *fit_LME_landmark*. The cause-specific model involves regression on the cause-specific hazard function. A covariate with a hazard rate greater than 1 is associated with an increased rate of the event of interest and less than 1 with a decreased rate of the event. Conversely, the Fine Gray model involves regression on the cumulative incidence function (CIF). In this case, a covariate with a hazard rate greater than 1 is associated with an increased incidence of the event of interest and less than 1 with a decreased incidence of the event.  Their differences has led to the proposal that the cause-specific model is preferred when the aim is to study the aetiology of a disease, and the Fine Gray model is preferred when the aim is prediction of the risk of an event.

### 3. Cross-validation

Another feature of this package is that is makes it easy to perform k-fold cross validation. The function *add_cv_number* assigns a fold to each individual in a dataset. When multiple landmark ages are defined within functions *fit_LOCF_landmark* or *fit_LME_landmark*, the cross validation folds are kept consistent for individuals across the corresponding landmark models.  


## How is the landmark model fit within the R package Landmarking?

The functions `fit_LOCF_landmark` and `fit_LME_landmark` are used to fit the landmark model in this package. See the vignette **How to use the R package 'Landmarking'** to see a worked example which uses these functions.

Here is a diagram that represents the functions `fit_LOCF_landmark` (selecting option `fit_LOCF_longitudinal`) and `fit_LME_landmark` (selecting option `fit_LME_longitudinal`) to show how they are made up of their helper functions. All these functions are available with the R package Landmarking. 

```{r figurename3, echo=FALSE, fig.cap=""}
knitr::include_graphics("../vignettes/fit_LOCF_landmark_model.png")
```

The first step of the functions `fit_LOCF_landmark` and `fit_LME_landmark` is forming the
datasets that correspond to each value of the landmark ages in `x_L`. These datasets will contain
only the data of the individuals in the risk set at each landmark age `x_L`. To be in the risk set at age `x_L` the
individual must have:

* entered the study at a time up to and including the landmark age `x_L`. This means that the individual
has at least one LOCF value for all variables in `covariate` for the case of `fit_LOCF_landmark`, or
at least one LOCF value for all variables in `predictors_LME` for the case of `fit_LME_landmark`.  This condition is necessary for the LOCF approach as the LOCF values at the landmark age `x_L` are extracted for all `covariates` and so at least one observation is needed of each of these. This condition is necessary for the LME approach, as the LOCF values at the landmark age `x_L` are extracted for `predictors_LME` and so at least one observation is needed of each of these. On the other hand, the function `fit_LME_landmark` allows for missing observations in `responses_LME` by the landmark age.

* exited the study after and not including time `x_L`.

This is performed using the functions `find_LOCF_risk_set` or `find_LME_risk_set`.

After this dataset is formed for each of the landmark ages, the second step involves adding the cross-validation fold to the individuals in the dataset. The datasets that were just formed are re-combined in order to ensure that the cross-validation fold assignment is consistent across each of the datasets. In other words the individual that is in the risk set at one landmark time has the same cross-validation number as when they are in the risk set at another landmark time. The cross-validation folds may be defined in two ways:

* the user sets parameter `k` which randomly assigns cross-validation folds to each individual using the function `add_cv_number`

* the user sets parameter `cross_validation_df` which refers to a data frame that contains the user-defined assignment of cross-validation folds.

Alternatively, no cross-validation is performed.

In the third and fourth steps the landmark model is fitted separately to each of the datasets that correspond to a landmark age `x_L` (with the `cross_validation_number` added). If the parameters `k` or `cross_validation_df` are used, then the landmark model is formed using the training dataset and applied to the test dataset. 

In the third step, function `fit_LOCF_longitudinal` or `fit_LME_longitudinal` is used. The function `fit_LOCF_longitudinal` extracts the LOCF value for each of the `covariates` in `data_long` up to (and including) time `x_L`. On the other hand, the function `fit_LME_longitudinal` fits a multivariate linear mixed effects model (with unstructured covariance) to the training dataset. The model contains a random intercept and random slope (if `random_slope_longitudinal=TRUE`). Once this model is fitted, the BLUP estimates of the response variables (and slopes if `random_slope_survival=TRUE` is used) are calculated for the test dataset. These values, along with the LOCF value for each of the covariates in `predictors_LME`, are used as the covariates in the survival model. For more information see `help(fit_LME_longitudinal)`.

Fitting the survival model is performed using `fit_survival_model`. This function censors the individuals at the horizon time `x_hor` and then fits the survival submodel specified in the function arguments. There are
three choices for this model (standard Cox model, Fine Gray model, and cause-specific model). For more information see `help(fit_survival_model)`.

In the fifth step, the performance of the model is assessed using c-index and Brier score. If the parameter `b` is defined, then the bootstrap estimates of the c-index and Brier score are calculated as well. For both the c-index and Brier score calculations, inverse probability censoring weighting (IPCW) is used to create weights which account for the occurrence of censoring. The censoring model assumes for this function is the Kaplan Meier model, i.e. censoring occurs independently of covariates.

---
title: "Introduction to Landmark Models and the R package Landmarking"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to Landmark Models and the R package Landmarking}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


## What is the landmark model used for?

Landmark models are used to **dynamically predict** the risk of an event for an individual. This means that an individual has a personalised risk prediction which is updated as new information is collected about them. Landmark models are particularly applicable in a medical setting where electronic health records (EHRs), such as GP records, are continually being updated with new data.

## What is the landmark model?

The key idea behind landmark models is the concept of predefined **landmark times**. A landmark time is a time point (i.e. "landmark") that we wish to make a risk prediction at. A model is built at the landmark time and this model is developed only using the data of individuals at risk (i.e. not censored or having experienced an event) at the landmark time. 

>For example, suppose that we want to predict the 5 year risk of a cardiovascular disease (CVD) event for a 45 year old patient. Then we would create a model for landmark time 45 years old using all the individuals that are in the risk set at 45 years old, and use this model to predict the risk of an event occurring for our new individual.

 As we want to predict the risk of an event for an individual at a selection of time points, we define multiple landmark times and develop a model for each of these. 
 
>Continuing the example and supposing we wish to be able to predict the risk of CVD disease for any age between 45 and 50 years old, we define six landmark ages (45, ... , 50 years old) and fit six models for each of these. This is demonstrated in Figure 1.

```{r figurename, echo=FALSE, fig.cap="*Figure 1: Illustration of a landmark model with six landmark times (ages 45 to 50 years). The blue solid line represents the time interval of longitudinal (repeat measures) data that feeds into the first stage of the two-stage landmark model. The red dashed line represents the time interval of the time-to-event data to be inputted into the survival model which is the second stage of the landmark model. Events after the time horizon need to be censored within this framework.*", out.width = '70%'}
knitr::include_graphics("./images/landmarking_models.png")
```

To form a model at each landmark age, landmark models use a **two-stage model** that is comprised of a extracting information from the **longitudinal data** which then feeds into **survival model**. The typical form of this is the last observation carried forward (LOCF) method and the Cox proportional hazards model respectively, although there are extensions and these are discussed in the `Features of the R package Landmarking' section. Figure 1 shows the time intervals of the longitudinal (repeat measures) data and time-to-event data that is included when using the model, which is relative to the landmark age and time horizon. It should be noted that all the longitudinal (repeat measures) data may be used, including those passed the landmark age, in the first stage to develop a model. The survival model is fit only for the time-to-event data between the landmark age and the time horizon with all individuals censored at the time horizon. For a Cox proportional hazard survival  model, this has the advantage that proportional hazards is only assumed for the short period between the landmark time and the time horizon rather than the entire period up until the end of study.

## What are the advantages of the landmark model?

The other main statistical framework for this dynamic prediction problem is joint modelling. In comparison to joint modelling, the landmarking model is less computationally intensive and makes a less strong assumption about proportional hazards. Specifically this assumption only applies to the period between the landmark time and time horizon when using the landmark model.

## Features of the R package Landmarking

The package **Landmarking** offers benefits over existing packages *dynpred* and *landpred* for landmark model analysis. In particular, three features of the package are outlined below.

### 1. Linear mixed effects modelling 

The LME model is an alternative to using the LOCF method for the longitudinal data. LME modelling means that the entire history of a risk factor is used to evaluate predicted risk, as opposed to the LOCF model where only one (the most recent) measurement is used. Moreover the LME allows a patient to have **missing values** for the variables in the random effects component.

Figure 2 illustrates the use of these two types of models to predict systolic blood pressure at the landmark time. The LOCF model simply uses the most recently observed value of a risk factor. On the other hand, the LME model uses the entire history of the values of the risk factor from the individual, in combination with the trend in the risk factor from individuals used to develop the model. Using the Landmarking package, it is possible include a covariance structure in the random effects meaning that other risk factors can inform predictions. This improves the accuracy of the estimated value of the risk factor at the landmark time value compared to the LOCF model.

```{r figurename2, echo=FALSE, fig.cap="*Figure 2: Comparison of the LOCF and LME model to predict systolic blood pressure at a landmark age for an individual with 8 repeat measures.*", out.width = '90%'}
knitr::include_graphics("./images/landmarking_lme_model.png")
```

### 2. Competing risks modelling

The Landmarking R package has the ability to fit competing risks models for the survival model. There are two types of competing risks model that can be fit using this package, the Fine Gray model and the cause-specific model, using the parameter *survival_submodel* in the function *fit_LOCF_landmark* or *fit_LME_landmark*. The cause-specific model involves regression on the cause-specific hazard function. A covariate with a hazard rate greater than 1 is associated with an increased rate of the event of interest and less than 1 with a decreased rate of the event. Conversely, the Fine Gray model involves regression on the cumulative incidence function (CIF). In this case, a covariate with a hazard rate greater than 1 is associated with an increased incidence of the event of interest and less than 1 with a decreased incidence of the event.  Their differences has led to the proposal that the cause-specific model is preferred when the aim is to study the aetiology of a disease, and the Fine Gray model is preferred when the aim is prediction of the risk of an event.

### 3. Cross-validation

Another feature of this package is that is makes it easy to perform k-fold cross validation. The function *add_cv_number* assigns a fold to each individual in a dataset. When multiple landmark ages are defined within functions *fit_LOCF_landmark* or *fit_LME_landmark*, the cross validation folds are kept consistent for individuals across the corresponding landmark models.  


## How is the landmark model fit within the R package Landmarking?

The functions `fit_LOCF_landmark` and `fit_LME_landmark` are used to fit the landmark model in this package. See the vignette ``How to use the R package 'Landmarking''' to see a worked example which uses these functions.

Here is a diagram that represents the functions `fit_LOCF_landmark` (selecting option `fit_LOCF_longitudinal`) and `fit_LME_landmark` (selecting option `fit_LME_longitudinal`) to show how they are made up of their helper functions. All these functions are available with the R package Landmarking. 

```{r figurename3, echo=FALSE, fig.cap="", out.width = '100%'}
knitr::include_graphics("./images/fit_LOCF_landmark_model.png")
```
The first step of the functions `fit_LOCF_landmark` and `fit_LME_landmark` is forming the
datasets that correspond to each value of the landmark ages in `x_L`. These datasets will contain
only the data of the individuals in the risk set at each landmark age `x_L`. To be in the risk set at age `x_L` the
individual must have:

* entered the study at a time up to and including the landmark age `x_L`. This means that the individual
has at least one LOCF value for all variables in `covariate` for the case of `fit_LOCF_landmark`, or
at least one LOCF value for all variables in `fixed_effects` for the case of `fit_LME_landmark`. This condition is necessary
for the LOCF approach as the LOCF values at the landmark age `x_L` are extracted for all `covariates` and so at least one observation is needed of each of these. This condition is necessary for the LME approach, as the LOCF values at the landmark age `x_L` are extracted for `fixed_effects` and so at least one observation is needed of each of these. On the other hand, the function `fit_LME_landmark` allows for missing observations in `random_effects` by the landmark age as a result of the LME model.The individuals in that satisfy this condition are selected using the function `return_ids_with_LOCF`.

* exited the study after and not including time `x_L`.

After this dataset is formed for each of the landmark ages, the second step involves adding the cross-validation fold to the individuals in the dataset. The datasets that were just formed are re-combined in order to ensure that the cross-validation fold assignment is consistent across each of the datasets. In other words the individual that is in the risk set at one landmark time has the same cross-validation number as when they are in the risk set at another landmark time. The cross-validation folds may be defined in two ways:

* the user sets parameter `k` which randomly assigns cross-validation folds to each individual using the function `add_cv_number`

* the user sets parameter `cross_validation_df` which refers to a data frame that contains the user-defined assignment of cross-validation folds.

Alternatively, no cross-validation is performed.

In the third and fourth steps the landmark model is fitted separately to each of the datasets that correspond to a landmark age `x_L` (with the `cross_validation_number` added). If the parameters `k` or `cross_validation_df` are used, then the landmark model is formed using the training dataset and applied to the test dataset. 

In the third step, function `fit_LOCF_longitudinal` or `fit_LME_longitudinal` is used. The function `fit_LOCF_longitudinal` extracts the LOCF value for each of the `covariates` in `data_long` up to (and including) time `x_L`. On the other hand, the function `fit_LME_longitudinal` fits a multivariate linear mixed effects model (with unstructured covariance) to the training dataset. The model contains a random intercept and random slope if `random_slope_in_LME=TRUE`. Once this model is fitted the estimates of the `random_effects` for the test dataset at the landmark age `x_L` are found. The LOCF value for each of the covariates in `fixed_effects` is used as the covariates at the landmark age `x_L` in the prediction. The BLUP estimates (and slopes if `random_slope_as_covariate=TRUE` is used) are then calculated. For more information see `help(fit_LME_longitudinal)`.

These values are then used in the next step: fitting the survival model using `fit_survival_model`. There are
three choices for this model (standard Cox model, Fine Gray model, and cause-specific model). For more information see `help(fit_survival_model)`.

In the fifth step, the performance of the model is assessed using c-index and Brier score. If the parameter `b` is defined, then the bootstrap estimates of the c-index and Brier score are calculated as well. For both the c-index and Brier score calculations, inverse probability censoring weighting (IPCW) is used to create weights which account for the occurrence of censoring. The censoring model assumes for this function is the Kaplan Meier model, i.e. censoring occurs independently of covariates.

---
title: "How to use the R package 'Landmarking'"
author: Isobel Barrott
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{How to use the R package 'Landmarking'}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r 1,echo=FALSE}
knitr::opts_chunk$set(cache=FALSE,collapse=TRUE, comment="#>")
```

## Introduction

Below is a step-by-step demonstration of how to use this package. This example looks at using risk assessments recorded at primary care practices to predict the 5-year risk of cardiovascular disease (CVD).

We will perform the landmarking analysis using both the LOCF and LME methods and compare the results of these two models. The example involves competing risks events in the form of the risk of death from other causes; the cause-specific model will therefore be used as the survival submodel. For more information about the landmark model please see isobelbarrott.github.io/Landmarking.

## Exploring the datasets

First of all, let's take a look at the dataset we are going to analyse. This dataset is a combination of two datasets: one of these contains the longitudinal (repeat measures) data and one of these contains the time-to-event data.

* The longitudinal (repeat measurements) data from CVD risk assessments is in long format. Each row corresponds to a different assessment and an individual may have multiple assessments. The dataset contains information about an individual's ethnicity, smoking status, diabetes, and standardised systolic blood pressure (SBP). These were recorded at the patient age specified in `response_time_sbp_stnd`. The standardised total cholesterol to high density lipoprotein (HDL) ratio was also recorded as part of the assessment but was recorded at the patient age specified in `response_time_tchdl_stnd`, which is up to a month after the other variables were measured.

* The time-to-event data indicates the time and status of event that each individual experiences. There are three types of events: censoring (this is due to loss to follow-up or the end of the study, indicated by status `0`), CVD event (indicated by status `1`), or death from other causes (indicated by status `2`). It is important when using this package that 0 is used to indicate censoring, 1 to indicate the event of interest, and values 2 or above indicate different competing risks.

```{r 2}
library(Landmarking)
set.seed(1)
data(data_repeat_outcomes)
head(data_repeat_outcomes)
```

Looking further at these datasets we see that there are `r length(unique(data_repeat_outcomes$id))` patients in this study, each with an average of `r mean(table(data_repeat_outcomes$id))` assessments.

```{r 3}
length(unique(data_repeat_outcomes$id))
mean(table(data_repeat_outcomes$id))
```

Moreover, we can see the breakdown of events

```{r 4}
table(unique(data_repeat_outcomes[,c("id","event_status")])[,"event_status"])
```

So `r length(which(unique(data_repeat_outcomes[,c("id","event_status")])[,"event_status"]==1))` patients in the study (out of `r length(unique(data_repeat_outcomes$id))`) experienced a CVD event and `r length(which(unique(data_repeat_outcomes[,c("id","event_status")])[,"event_status"]==2))` experienced death from other causes.


## Analysis
We are now ready to begin the analysis using the Landmarking package.

There are a couple of issues to deal with before we start, these stem from the fact that we wish to compare the LOCF and LME longitudinal model. We want to ensure that the same dataset is used for both of these models so that the comparison is fair. 

The first issue is that the presence of `NA` values means that some individuals in the risk set do not have LOCF values for all the covariates. This means that the LOCF longitudinal model cannot be fitted for these individuals, however the LME longitudinal model can allow missing values in the random effects covariates. As we wish that the dataset that both models are fitted are the same, we should consider removing these individuals from the dataset.

Helpfully, the `Landmarking` package contains a function `return_ids_with_LOCF` to create a dataset with individuals with a LOCF for all covariates contained in the `covariates` parameter at landmark time `x_L`.

```{r 9}
data_repeat_outcomes <-
  return_ids_with_LOCF(
    data_long = data_repeat_outcomes,
    individual_id = "id",
    covariates = c("ethnicity", "smoking", "diabetes", "sbp_stnd", "tchdl_stnd"),
    covariates_time = c(rep("response_time_sbp_stnd", 4), "response_time_tchdl_stnd"),
    x_L = c(60, 61)
  )
```

The second issue is to do with cross-validation, but we will deal with this once we have fitted the LOCF landmark model.

So we can now  fit the landmark model using the LOCF longitudinal submodel. To demonstrate how to use the package, we use only landmark times 60 and 61 years old however we may wish to use more landmark times in practice. For more information about this function, see the Details section of the `fit_LOCF_landmark_model` documentation.


```{r 11}
data_model_landmark_LOCF <-
  fit_LOCF_landmark(
    data_long = data_repeat_outcomes,
    x_L = c(60, 61),
    x_hor = c(65, 66),
    covariates = c("ethnicity", "smoking", "diabetes", "sbp_stnd", "tchdl_stnd"),
    covariates_time = c(rep("response_time_sbp_stnd", 4), "response_time_tchdl_stnd"),
    k = 10,
    individual_id = "id",
    event_time = "event_time",
    event_status = "event_status",
    survival_submodel = "cause_specific"
  )
```

We can see that fitting this model returns the warnings such as `Loglik converged before variable  2 ; coefficient may be infinite.`. Taking a look at the model fit contained in `data_model_landmark_LOCF$model_survival`, we can see the confidence intervals for certain folds is very large. This is due there being few observations for these covariates. However, as only the point estimate of the coefficient is needed to calculate the risk prediction, inaccurate standard error values is not a problem. Indeed we have obtained the risk predictions and can take a look at their distribution.


```{r 12}
plot(
  density(100 * data_model_landmark_LOCF[["60"]]$data$event_prediction),
  xlab = "Predicted risk of CVD event (%)",
  main = "Landmark age 60"
)
```

Printing the object that has been outputted returns a list of datasets corresponding to each of the landmark groups. These contain the column `event_prediction` which indicates the risk prediction of the event of interest.

```{r 13}
data_model_landmark_LOCF
```

The output also contains the prediction error for the model, which is assessed using the Brier score and C-index. The standard error for these estimates can be calculated by setting parameter `b` to the desired number of bootstrap samples. 

```{r 14}
data_model_landmark_LOCF[["60"]]$prediction_error
data_model_landmark_LOCF[["61"]]$prediction_error
```

In order to assess the performance of the model, there is also the `plot` method which creates a calibration plot which can assess the calibration of the model. The perfect prediction model will have points along the y=x line and deviations from this suggest poor calibration.

```{r 20}
plot(x = data_model_landmark_LOCF, x_L = 60, n = 5)
```

Now we can move on to fitting the landmark model using the LME longitudinal submodel.  For more information about this function, see the Details section of the `fit_LME_landmark_model` documentation. The variables standardised systolic blood pressure (SBP) and standardised total cholesterol to high density lipoprotein (HDL) ratio are continuous and repeatedly measured and so these are included as random effects in the LME model.  

The second issue which was mentioned previously is related to cross-validation. In particular we wish to perform k-fold cross-validation using exactly the same folds for both LOCF and LME models to ensure a fair comparison. To do this we use the parameter `cross_validation_df` which uses a list of data frames that indicate which fold each individual belongs to. We create this list using the output of `fit_LOCF_landmark_model`.

```{r 15}
cross_validation_list <- lapply(data_model_landmark_LOCF, "[[", i = 1)
```

There are a couple of computational considerations to highlight about fitting the LME model. The parameter `lme_control` can be altered in order to change the  default values of parameters (such as maximum number of iterations) for the algorithm which fits the LME model. Also setting `standardise_time = TRUE` standardises the values of the time variable and this helps fitting the LME model (see `?fit_LME_landmark_model`).

We fit the LME landmark model in a similar way to the LOCF landmark model, but with a few extra parameters. Fitting the LME model can take some time so here we only use landmark age 60 for demonstration purposes. 

```{r 16}
data_model_landmark_LME <-
  fit_LME_landmark(
    data_long = data_repeat_outcomes[["60"]],
    x_L = c(60),
    x_hor = c(65),
    cross_validation_df =
      cross_validation_list,
    fixed_effects = c("ethnicity", "smoking", "diabetes"),
    fixed_effects_time =
      "response_time_sbp_stnd",
    random_effects = c("sbp_stnd", "tchdl_stnd"),
    random_effects_time = c("response_time_sbp_stnd", "response_time_tchdl_stnd"),
    individual_id = "id",
    standardise_time = TRUE,
    lme_control = nlme::lmeControl(maxIter =
                                     100, msMaxIter = 100),
    event_time = "event_time",
    event_status = "event_status",
    survival_submodel = "cause_specific"
  )
```

Comparing the prediction error for the landmarking with LOCF and LME, we can see a slight improvement when the LME model is used.

```{r 17}
data_model_landmark_LOCF[["60"]]$prediction_error
data_model_landmark_LME[["60"]]$prediction_error
```

Finally, suppose that we wish to predict the risk of the event of interest for a new patient. We need to create a data frame that contains records of all the covariates that were used to form the landmark model. As cross-validation was used, it is necessary to specify which cross-validation fold to use in the prediction. It is possible to predict the risk of the event of interest at any time before the horizon time.

```{r 18}
newdata <-
  data.frame(
    id = c(3001, 3001, 3001),
    response_time_sbp_stnd = c(57, 58, 59),
    smoking = c(0, 0, 0),
    diabetes = c(0, 0, 0),
    ethnicity = c("Indian", "Indian", "Indian"),
    sbp_stnd = c(0.45, 0.87, 0.85),
    tchdl_stnd = c(-0.7, 0.24, 0.3),
    response_time_tchdl_stnd = c(57, 58, 59)
  )
predict(
  object = data_model_landmark_LME,
  x_L = 60,
  x_hor = 62,
  newdata = newdata,
  cv_fold = 1
)
predict(
  object = data_model_landmark_LME,
  x_L = 60,
  x_hor = 64,
  newdata = newdata,
  cv_fold = 1
)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.landmark.R
\name{predict.landmark}
\alias{predict.landmark}
\title{Predict the risk of an event for a new individual using the landmark model}
\usage{
\method{predict}{landmark}(object, x_L, x_hor, newdata, cv_fold = NA, ...)
}
\arguments{
\item{object}{Object inheriting the class \code{landmark}, this should be the output from either \code{fit_LME_landmark} or \code{fit_LOCF_landmark}. It should contain a list
of landmark models corresponding to different landmark times \code{x_L}.}

\item{x_L}{Numeric specifying the landmark time. This indicates which landmark model in \code{object} to use.}

\item{x_hor}{Numeric specifying the horizon time. The function assesses the risk of event before this time.}

\item{newdata}{Data frame containing new data to return the risk prediction of the event of interest. The data should be in in long format
and the columns must contain the covariates and time variables that are used to fit the model.
For the LME model this the variables \code{predictors_LME}, \code{responses_LME}, \code{predictors_LME_time}, and
\code{responses_LME_time}. For the LOCF model this is \code{covariates} and \code{covariates_time}.}

\item{cv_fold}{If cross validation is used to fit \code{fit_LME_landmark} or \code{fit_LOCF_landmark}, then the cross validation fold to use when making risk predictions needs to be specified.}

\item{\dots}{Arguments passed on to \code{riskRegression::predictRisk}}
}
\value{
Data frame \code{newdata} updated to contained a new column \code{event_prediction}
}
\description{
This function predicts the risk of an event for new data using the landmark model fitted by \code{fit_LME_landmark} or \code{fit_LOCF_landmark}.
The 'event' is defined as event for which \code{event_status} is 1.
}
\examples{
library(Landmarking)
data(data_repeat_outcomes)
data_model_landmark_LOCF <-
  fit_LOCF_landmark(
    data_long = data_repeat_outcomes,
    x_L = c(60, 61),
    x_hor = c(65, 66),
    covariates =
      c("ethnicity", "smoking", "diabetes", "sbp_stnd", "tchdl_stnd"),
    covariates_time =
      c(rep("response_time_sbp_stnd", 4), "response_time_tchdl_stnd"),
    k = 10,
    individual_id = "id",
    event_time = "event_time",
    event_status = "event_status",
    survival_submodel = "cause_specific"
  )
 newdata <-
   rbind(
     data.frame(
       id = c(3001, 3001, 3001),
       response_time_sbp_stnd = c(57, 58, 59),
       smoking = c(0, 0, 0),
       diabetes = c(0, 0, 0),
       ethnicity = c("Indian", "Indian", "Indian"),
       sbp_stnd = c(0.45, 0.87, 0.85),
       tchdl_stnd = c(-0.7, 0.24, 0.3),
       response_time_tchdl_stnd = c(57, 58, 59)
     )
   )
 predict(object=data_model_landmark_LOCF,x_L=60,x_hor=62,newdata=newdata,cv_fold=1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_LOCF_landmark.R
\name{fit_LOCF_longitudinal}
\alias{fit_LOCF_longitudinal}
\title{Find the last observation carried forward (LOCF) values for covariates in a dataset}
\usage{
fit_LOCF_longitudinal(
  data_long,
  x_L,
  covariates,
  covariates_time,
  cv_name = NA,
  individual_id
)
}
\arguments{
\item{data_long}{Data frame in long format i.e. there may be more than one row per individual}

\item{x_L}{Numeric specifying the landmark time(s)}

\item{covariates}{Vector of character strings specifying the column names in \code{data_long}
which correspond to the covariates}

\item{covariates_time}{Vector of character strings specifying the column names in \code{data_long}
which contain the times at which \code{covariates} were recorded. This should either be length 1 or
the same length as \code{covariates}. In the latter case the order of elements must correspond to the
order of elements in \code{covariates}.}

\item{cv_name}{Character string specifying the column name in \code{data_long} that indicates cross-validation fold}

\item{individual_id}{Character string specifying the column name in \code{data_long} which contains the individual identifiers}
}
\value{
List containing \code{data_longitudinal}, \code{model_longitudinal}, and \code{call}.

\code{data_longitudinal} has one row for each individual in \code{data_long} and
contains the LOCF value of \code{covariates} at the landmark time \code{x_L}.

\code{model_longitudinal} indicates that the LOCF approach is used.

\code{call} contains the call of the function.
}
\description{
This function is a helper function for \code{fit_LOCF_landmark}.
}
\details{
This function extracts the LOCF value for each of the \code{covariates} in \code{data_long} up to (and including) time \code{x_L}.
}
\author{
Isobel Barrott \email{isobel.barrott@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.landmark.R
\name{plot.landmark}
\alias{plot.landmark}
\title{Create a calibration plot}
\usage{
\method{plot}{landmark}(x, x_L, n, x_lims, y_lims, ...)
}
\arguments{
\item{x}{Object inheriting the class \code{landmark}, this should be the output from either \code{fit_LME_landmark_model} or \code{fit_LOCF_landmark_model}. It should contain a list
of landmark models corresponding to different landmark times \code{x_L}.}

\item{x_L}{Numeric specifying the landmark time. This indicates which landmark model in \code{x} to use.}

\item{n}{Numeric specifying the number of bins to use.}

\item{x_lims}{Vector of length 2 specifying the limits of the x axes}

\item{y_lims}{Vector of length 2 specifying the limits of the y axes}

\item{\dots}{Arguments passed to \code{ggplot2::labs} to modify axis, legend, and plot labels}
}
\value{
Calibration plot showing the value of predicted probabilities against observed frequencies, with a \code{y=x} line.
}
\description{
Creates a calibration plot for the landmark model fitted by \code{fit_LME_landmark_model} or \code{fit_LOCF_landmark_model}.
This function plots the observed frequencies of the event of interest against the predicted probabilities of the event of interest.
}
\details{
This function bins the predicted probabilities of the event of interest into \code{n} bins. The event of interest is the event with
\code{event_status=1} when fitting the landmark model. For each of the \code{n} sets of individuals, the Aalen-Johansen estimator is fit to that set
and used to calculate the risk of an event at the horizon time. The predictions (from the landmark model) and the observed frequencies
(from the Aalen-Johansen estimator) are plotted against each other. For a perfect prediction model, the points will be plotted along the y=x line.
}
\examples{
library(Landmarking)
data(data_repeat_outcomes)
data_model_landmark_LOCF <-
  fit_LOCF_landmark(
    data_long = data_repeat_outcomes,
    x_L = c(60, 61),
    x_hor = c(65, 66),
    covariates =
      c("ethnicity", "smoking", "diabetes", "sbp_stnd", "tchdl_stnd"),
    covariates_time =
      c(rep("response_time_sbp_stnd", 4), "response_time_tchdl_stnd"),
    k = 10,
    individual_id = "id",
    event_time = "event_time",
    event_status = "event_status",
    survival_submodel = "cause_specific"
  )
 plot(x=data_model_landmark_LOCF,x_L=60,n=5)
 plot(x=data_model_landmark_LOCF,x_L=61,n=5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_LME_landmark.R
\name{fit_LME_landmark}
\alias{fit_LME_landmark}
\title{Fit a landmarking model using a linear mixed effects (LME) model for the longitudinal submodel}
\usage{
fit_LME_landmark(
  data_long,
  x_L,
  x_hor,
  predictors_LME,
  responses_LME,
  predictors_LME_time,
  responses_LME_time,
  random_slope_longitudinal = TRUE,
  random_slope_survival = TRUE,
  include_data_after_x_L = TRUE,
  individual_id,
  k,
  cross_validation_df,
  standardise_time = FALSE,
  lme_control = nlme::lmeControl(),
  event_time,
  event_status,
  survival_submodel = c("standard_cox", "cause_specific", "fine_gray"),
  b
)
}
\arguments{
\item{data_long}{Data frame or list of data frames each corresponding to a landmark age \code{x_L} (each element of the list must be named the value of \code{x_L} it corresponds to).
Each data frame contains repeat measurements data and time-to-event data in long format.}

\item{x_L}{Numeric specifying the landmark time(s)}

\item{x_hor}{Numeric specifying the horizon time(s)}

\item{predictors_LME}{Vector of character strings specifying the column names in \code{data_long}
which correspond to the predictor variables in the LME model}

\item{responses_LME}{Vector of character strings specifying the column names in \code{data_long}
which correspond to the response variables in the LME model}

\item{predictors_LME_time}{Vector of character strings specifying the column names in \code{data_long}
which contains the time at which the predictor variables were recorded.
This should either be length 1 or the same length as \code{predictors_LME}.
In the latter case the order of elements must correspond to the order of elements in \code{predictors_LME}.}

\item{responses_LME_time}{Vector of character strings specifying the column names in \code{data_long}
which contain the times at which response variables were recorded. This should either be length 1 or
the same length as \code{responses_LME}. In the latter case the order of elements must correspond to the
order of elements in \code{responses_LME}.}

\item{random_slope_longitudinal}{Boolean indicating whether to include a random slope in the LME model. See Details section of \code{fit_LME_longitudinal} for more information.}

\item{random_slope_survival}{Boolean indicating whether to include the random slope estimate from the LME model.
See Details section of \code{fit_LME_longitudinal} for more information.
as a covariate in the survival submodel. See Details section of \code{fit_LME_longitudinal} for more information.}

\item{include_data_after_x_L}{Boolean indicating whether to include all longitudinal data, including data after the landmark age \code{x_L},
in the model development dataset. See Details section of \code{fit_LME_longitudinal} for more information.}

\item{individual_id}{Character string specifying the column name in \code{data_long} which contains the individual identifiers}

\item{k}{Integer specifying the number of folds for cross-validation.
An alternative to setting parameter \code{cross_validation_df} for performing cross-validation; if both are missing no cross-validation is used.}

\item{cross_validation_df}{List of data frames containing the cross-validation fold each individual is assigned to. Each data frame in the list should be
named according to the landmark time \code{x_L} that they correspond. Each data frame should contain the columns \code{individual_id} and a column \code{cross_validation_number}
which contains the cross-validation fold of the individual. An alternative to setting parameter \code{k} for performing cross-validation;
if both are missing no cross-validation is used.}

\item{standardise_time}{Boolean indicating whether to standardise the time variable in the LME model by subtracting the mean
and dividing by the standard deviation. See Details section of \code{fit_LME_longitudinal} for more information.}

\item{lme_control}{Object created using \code{nlme::lmeControl()}, which will be passed to the \code{control} argument of the \code{lme}
function}

\item{event_time}{Character string specifying the column name in \code{data_long}
which contains the event time}

\item{event_status}{Character string specifying the column name in \code{data_long}
which contains the event status (where 0=censoring, 1=event of interest, if there are competing events these are labelled
2 or above).}

\item{survival_submodel}{Character string specifying which survival submodel to
use. Three options: the standard Cox model i.e. no competing risks (\code{"standard_cox"}),
the cause-specific regression model (\code{"cause_specific"}), or the Fine Gray
regression model (\code{"fine_gray"})}

\item{b}{Integer specifying the number of bootstrap samples to take when calculating standard error of c-index and Brier score}
}
\value{
List containing containing information about the landmark model at each of the landmark times.
Each element of this list is named the corresponding landmark time, and is itself a list containing elements:
\code{data}, \code{model_longitudinal}, \code{model_LME}, \code{model_LME_standardise_time}, \code{model_survival}, and \code{prediction_error}.

\code{data} has one row for each individual in the risk set at \code{x_L} and
contains the value of the \code{predictors_LME} using the LOCF approach and predicted values of the
\code{responses_LME} using the LME model at the landmark time \code{x_L}. It also includes the predicted
probability that the event of interest has occurred by time \code{x_hor}, labelled as \code{"event_prediction"}.
There is one row for each individual.

\code{model_longitudinal} indicates that the longitudinal approach is LME.

\code{model_LME} contains the output from
the \code{lme} function from package \code{nlme}. For a model using cross-validation,
\code{model_LME} contains a list of outputs with each
element in the list corresponds to a different cross-validation fold.

\code{model_LME_standardise_time} contains a list of two objects \code{mean_response_time} and \code{sd_response_time} if the parameter \code{standardise_time=TRUE} is used. This
is the mean and standard deviation use to normalise times when fitting the LME model.

\code{model_survival} contains the outputs from
the survival submodel functions, including the estimated parameters of the model. For a model using cross-validation,
\code{model_survival} will contain a list of outputs with each
element in the list corresponding to a different cross-validation fold.

\code{prediction_error} contains a list indicating the c-index and Brier score at time \code{x_hor} and their standard errors if parameter \code{b} is used.
}
\description{
This function performs the two-stage landmarking analysis.
}
\details{
Firstly, this function selects the individuals in the risk set at the landmark time \code{x_L}.
Specifically, the individuals in the risk set are those that have entered the study before the landmark time \code{x_L}
(there is at least one observation for each of the \code{predictors_LME} and \code{responses_LME} on or before \code{x_L}) and
exited the study after the landmark age (\code{event_time}
is greater than \code{x_L}).

Secondly, if the option to use cross validation
is selected (using either parameter \code{k} or \code{cross_validation_df}), then an extra column \code{cross_validation_number} is added with the
cross-validation folds. If parameter \code{k} is used, then the function \code{add_cv_number}
randomly assigns these folds. For more details on this function see \code{?add_cv_number}.
If the parameter \code{cross_validation_df} is used, then the folds specified in this data frame are added.
If cross-validation is not selected then the landmark model is
fit to the entire group of individuals in the risk set (this is both the training and test dataset).

Thirdly, the landmark model is then fit to each of the training datasets. There are two parts to fitting the landmark model: using the longitudinal data and using the survival data.
Using the longitudinal data is the first stage and is performed using \code{fit_LME_longitudinal}. See \code{?fit_LME_longitudinal} more for information about this function.
Using the survival data is the second stage and is performed using \code{fit_survival_model}. This function censors the
individuals at the time horizon \code{x_L} and fits the survival model. See \code{?fit_survival_model} more for information about this function.

Fourthly, the performance of the model is then assessed on the set of predictions
from the entire set of individuals in the risk set by calculating Brier score and C-index.
This is performed using \code{get_model_assessment}. See \code{?get_model_assessment} more for information about this function.
}
\examples{
\donttest{
library(Landmarking)
data(data_repeat_outcomes)
data_model_landmark_LME <-
  fit_LME_landmark(
    data_long = data_repeat_outcomes,
    x_L = c(60, 61),
    x_hor = c(65, 66),
    k = 10,
    predictors_LME = c("ethnicity", "smoking", "diabetes"),
    predictors_LME_time = "response_time_sbp_stnd",
    responses_LME = c("sbp_stnd", "tchdl_stnd"),
    responses_LME_time = c("response_time_sbp_stnd", "response_time_tchdl_stnd"),
    individual_id = "id",
    standardise_time = TRUE,
    lme_control = nlme::lmeControl(maxIter = 100, msMaxIter = 100),
    event_time = "event_time",
    event_status = "event_status",
    survival_submodel = "cause_specific"
  )
}
}
\author{
Isobel Barrott \email{isobel.barrott@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_cv_number.R
\name{add_cv_number}
\alias{add_cv_number}
\title{Assign a k-fold cross-validation number}
\usage{
add_cv_number(data_long, individual_id, k, seed = 1)
}
\arguments{
\item{data_long}{Data frame in long format i.e. there may be more than one row per individual}

\item{individual_id}{Character string specifying the column name in \code{data_long} which contains the individual identifiers}

\item{k}{Integer specifying the number of folds for cross-validation.}

\item{seed}{The value of the seed (default is 1)}
}
\value{
Data frame \code{data_long} updated to contain a new column \code{cross_validation_number}
indicating the fold to which the individual has been assigned.
}
\description{
Randomly assigns a k-fold cross-validation number to each individual in a dataset.
}
\details{
This function randomly divides the \code{n} individual IDs
into \code{k} groups, each with \code{n/k} members (or as close to this number as possible).
}
\examples{
data(data_repeat_outcomes)
data_repeat_outcomes <- add_cv_number(data_long = data_repeat_outcomes,
                                      individual_id = "id",
                                      k = 10)
}
\author{
Isobel Barrott \email{isobel.barrott@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_repeat_outcomes}
\alias{data_repeat_outcomes}
\title{Simulated repeat measurement and time-to-event data}
\format{
A dataset with 9031 rows and 12 columns:
\describe{
\item{id}{Patient ID}
\item{smoking}{Smoking status, 0 indicates the patient has never smoked, 1 indicates the patient has quit smoking, and 2 indicates the patient is a current smoker}
\item{diabetes}{Diabetes status, 0 indicates the patient is not diagnosed with diabetes, and 1 indicates the patient is diagnosed with diabetes}
\item{ethnicity}{Ethnicity, one of five ethnicities}
\item{dob}{Date of birth}
\item{index}{An index indicating assessment number for a patient}
\item{sbp_stnd}{Standardised systolic blood pressure}
\item{tchdl_stnd}{Standardised total cholesterol to HDL ratio}
\item{response_time_tchdl_stnd}{Age that total cholesterol to HDL ratio was recorded}
\item{response_time_sbp_stnd}{Age that systolic blood pressure was recorded, this is the same as the date that the fixed measures were recorded}
\item{event_status}{Event status, 0 indicates censoring, 1 indicates CVD event, and 2 indicates death from other causes}
\item{event_time}{Event time}
}
}
\usage{
data_repeat_outcomes
}
\description{
A simulated dataset which is a combination of longitudinal (repeat measurement) data and
time-to-event data. This dataset contains simulated data
from 3000 patients.

The longitudinal (repeat measurement) data is formed using an LME model,
whose parameters were based on CVD risk assessments
recorded at primary care practices in New Zealand.

A LME model was fitted to this dataset and values of \code{sbp_stnd}
and \code{tchdl_stnd} were estimated at landmark age 60. These values (along with the other baseline covariates)
were used to simulate time-to-event data from a
cause specific model with parameters based on CVD events of patients
at primary care practices in New Zealand.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/return_ids_with_LOCF.R
\name{return_ids_with_LOCF}
\alias{return_ids_with_LOCF}
\title{Select individuals in a dataset with a last observation carried forward (LOCF) at a landmark time}
\usage{
return_ids_with_LOCF(
  data_long,
  individual_id,
  x_L,
  covariates,
  covariates_time
)
}
\arguments{
\item{data_long}{Data frame with repeated measurements data in long format}

\item{individual_id}{Character string specifying the column name in \code{data_long} which contains the individual identifiers}

\item{x_L}{Numeric specifying the landmark time(s)}

\item{covariates}{Vector of character strings specifying the column names in \code{data_long}
which correspond to the covariates}

\item{covariates_time}{Vector of character strings specifying the column names in \code{data_long}
which contain the times at which \code{covariates} were recorded. This should either be length 1 or
the same length as \code{covariates}. In the latter case the order of elements must correspond to the
order of elements in \code{covariates}.}
}
\value{
List of data frames which correspond to each landmark time \code{x_L}.
Each data frame is an updated version of \code{data_long} which contains only rows
of individuals with a LOCF at age \code{x_L}, other rows are removed.
}
\description{
To fit the LOCF model, all individuals must have at least one
non-\code{NA} entry by landmark time \code{x_L} for all covariates.
This function selects these individuals and removes the other rows.
}
\details{
Individuals have a LOCF if there is a non-\code{NA} entry for each of the covariates in
\code{covariates} up until (not including) time \code{x_L}.
}
\examples{
library(Landmarking)
data(data_repeat_outcomes)
data_repeat_outcomes <-
  return_ids_with_LOCF(
    data_long = data_repeat_outcomes,
    individual_id = "id",
    covariates =
      c("ethnicity", "smoking", "diabetes", "sbp_stnd", "tchdl_stnd"),
    covariates_time =
      c(rep("response_time_sbp_stnd", 4), "response_time_tchdl_stnd"),
    x_L = c(60,61)
  )
}
\author{
Isobel Barrott \email{isobel.barrott@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixoutsamp.R
\name{mixoutsamp}
\alias{mixoutsamp}
\title{Calculate point estimates from a linear mixed effects (LME) model for new data}
\usage{
mixoutsamp(model, newdata)
}
\arguments{
\item{model}{Object of class \code{lme} containing the fitted LME model}

\item{newdata}{Data frame containing data for which to make predictions.
The response variable should be set to NA for the rows of the data the user wishes to make predictions for.
The columns in the data should have the same names as those used to fit the model.
The variables should also be of the same type as in the data used to fit the mixed model (numeric, factor etc).}
}
\value{
List containing \code{preddata} and \code{random}. Data frame \code{preddata} is a version of \code{newdata} updated to contain columns corresponding to the fixed effects values (\code{fixed}),
random effects values (\code{random}), and fitted values (\code{fitted}).
Data frame \code{random} contains the values of random effects components for each individual.
}
\description{
This function allows the user to make out-of-sample predictions from an LME model.
}
\author{
This code was originally written by Ruth Keogh (London School of Hygiene and Tropical Medicine) which can be viewed at
github.com/ruthkeogh/landmark_CF. There have been further contributions from Jessica Barrett (MRC Biostatistics Unit, University of Cambridge),
David Stevens (University of Liverpool), and Mike Sweeting (University of Leicester).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_model_assessment.R
\name{get_model_assessment}
\alias{get_model_assessment}
\title{Compute C-index and Brier score}
\usage{
get_model_assessment(
  data,
  individual_id,
  event_prediction,
  event_status,
  event_time,
  x_hor,
  b
)
}
\arguments{
\item{data}{Data frame containing survival outcomes and the event predictions from the model,
there should be one row for each individual}

\item{individual_id}{Character string specifying the column name in \code{data} which contains the individual identifiers}

\item{event_prediction}{Character string specifying the column name in \code{data} containing the predicted probability of the event of interest}

\item{event_status}{Character string specifying the column name in \code{data} which contains the event status (where 0=censoring, 1=event of interest, if there are competing events these are labelled 2 or above). Events at time \code{x_hor} should be labelled censored.}

\item{event_time}{Character string specifying the column name in \code{data} which contains the event time.}

\item{x_hor}{Numeric specifying the horizon time(s)}

\item{b}{Integer specifying the number of bootstrap samples to take when calculating standard error of c-index and Brier score}
}
\value{
List containing C-index, Brier score and their standard errors
}
\description{
Performs model assessment by computing the C-index and Brier score at time \code{x_hor}. There is the option
to calculate their standard errors using bootstraping.
}
\details{
There are two factors in assessing the performance of a prediction model; its
discrimination and its calibration. The c-index is a commonly used metric which assesses
discrimination, this refers to the ability of the model to separate individuals into
those that will have an event and those that will not. The c-index at a horizon time \code{x_hor}
looks at the pairs of individuals where one individual has the event at a time T and the other has not had the event at time T.
It is calculated as the proportion of these pairs where their relative risk prediction agrees with the
actual outcomes for the two individuals. This is extended to the competing risks case
by comparing individuals where one had the event of interest at time T and the other individual either
did not experience the event before this time T or experienced a competing event.

The Brier score gives an indication of the calibration of a model (and its discrimination to an extent), this refers to the agreement between the risk prediction and
the outcome. The Brier score is calculated as the average mean squared error of the predicted risk and the event outcome (where
an event is 1 and not experiencing the event is 0). This is extended to the competing risks case by including the competing risk events as
not experiencing the event.

For both the c-index and Brier score calculations, inverse probability censoring weighting (IPCW) is used to create weights
which account for the occurrence of censoring. The censoring model assumes for this function is the Kaplan Meier model, i.e. censoring occurs
independently of covariates.

The c-index is calculated using the \code{cindex} function in package \code{pec}. The Brier score is calculated using
\code{pec} function in package \code{pec}.
}
\examples{
\dontrun{
library(Landmarking)
data(data_repeat_outcomes)
data_model_landmark_LOCF <-
  fit_LOCF_landmark(
    data_long = data_repeat_outcomes,
    x_L = c(60, 61),
    x_hor = c(65, 66),
    covariates =
      c("ethnicity", "smoking", "diabetes", "sbp_stnd", "tchdl_stnd"),
    covariates_time =
      c(rep("response_time_sbp_stnd", 4), "response_time_tchdl_stnd"),
    k = 10,
    individual_id = "id",
    event_time = "event_time",
    event_status = "event_status",
    survival_submodel = "cause_specific"
  )
get_model_assessment(data = data_model_landmark_LOCF[["60"]]$data,
  individual_id = "id",
  event_prediction = "event_prediction",
  event_status = "event_status",
  event_time = "event_time",
  x_hor = 65,
  b = 100)}
}
\author{
Isobel Barrott \email{isobel.barrott@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_LME_landmark.R
\name{find_LME_risk_set}
\alias{find_LME_risk_set}
\title{Find the risk set for a landmark model (LME)}
\usage{
find_LME_risk_set(
  data_long,
  x_L,
  x_hor,
  predictors_LME,
  predictors_LME_time,
  responses_LME,
  responses_LME_time,
  individual_id,
  event_time,
  event_status
)
}
\arguments{
\item{data_long}{Data frame in long format i.e. there may be more than one row per individual}

\item{x_L}{Numeric specifying the landmark time(s)}

\item{x_hor}{Numeric specifying the horizon time(s)}

\item{predictors_LME}{Vector of character strings specifying the column names in \code{data_long}
which correspond to the predictor variables in the LME model}

\item{predictors_LME_time}{Vector of character strings specifying the column names in \code{data_long}
which contains the time at which the predictor variables were recorded.
This should either be length 1 or the same length as \code{predictors_LME}.
In the latter case the order of elements must correspond to the order of elements in \code{predictors_LME}.}

\item{responses_LME}{Vector of character strings specifying the column names in \code{data_long}
which correspond to the response variables in the LME model}

\item{responses_LME_time}{Vector of character strings specifying the column names in \code{data_long}
which contain the times at which response variables were recorded. This should either be length 1 or
the same length as \code{responses_LME}. In the latter case the order of elements must correspond to the
order of elements in \code{responses_LME}.}

\item{individual_id}{Character string specifying the column name in \code{data_long} which contains the individual identifiers}

\item{event_time}{Character string specifying the column name in \code{data_long}
which contains the event time}

\item{event_status}{Character string specifying the column name in \code{data_long}
which contains the event status (where 0=censoring, 1=event of interest, if there are competing events these are labelled
2 or above).}
}
\value{
List with elements corresponding to each landmark time in x_L. Each element is a data frame, containing only those individuals
in the risk set at each of the landmark times x_L.
}
\description{
This function is a helper function for \code{fit_LME_landmark}.
}
\details{
This function finds the risk set for each of landmark times in x_L. This means that each of the individuals has a LME value for all predictors_LME at the landmark time and
has not experienced an event up to (and including) the landmark time.
}
\author{
Isobel Barrott \email{isobel.barrott@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_LME_landmark.R
\name{fit_LME_longitudinal}
\alias{fit_LME_longitudinal}
\title{Fit a landmarking model using a linear mixed effects (LME) model for the longitudinal data}
\usage{
fit_LME_longitudinal(
  data_long,
  x_L,
  predictors_LME,
  responses_LME,
  predictors_LME_time,
  responses_LME_time,
  standardise_time = FALSE,
  random_slope_longitudinal = TRUE,
  random_slope_survival = TRUE,
  include_data_after_x_L = TRUE,
  cv_name = NA,
  individual_id,
  lme_control = nlme::lmeControl()
)
}
\arguments{
\item{data_long}{Data frame containing repeat measurement data and time-to-event data in long format.}

\item{x_L}{Numeric specifying the landmark time(s)}

\item{predictors_LME}{Vector of character strings specifying the column names in \code{data_long}
which correspond to the predictor variables in the LME model}

\item{responses_LME}{Vector of character strings specifying the column names in \code{data_long}
which correspond to the response variables in the LME model}

\item{predictors_LME_time}{Vector of character strings specifying the column names in \code{data_long}
which contains the time at which the predictor variables were recorded.
This should either be length 1 or the same length as \code{predictors_LME}.
In the latter case the order of elements must correspond to the order of elements in \code{predictors_LME}.}

\item{responses_LME_time}{Vector of character strings specifying the column names in \code{data_long}
which contain the times at which response variables were recorded. This should either be length 1 or
the same length as \code{responses_LME}. In the latter case the order of elements must correspond to the
order of elements in \code{responses_LME}.}

\item{standardise_time}{Boolean indicating whether to standardise the time variable in the LME model by subtracting the mean
and dividing by the standard deviation. See Details section of \code{fit_LME_longitudinal} for more information.}

\item{random_slope_longitudinal}{Boolean indicating whether to include a random slope in the LME model. See Details section of \code{fit_LME_longitudinal} for more information.}

\item{random_slope_survival}{Boolean indicating whether to include the random slope estimate from the LME model
as a covariate in the survival submodel. See Details section of \code{fit_LME_longitudinal} for more information.}

\item{include_data_after_x_L}{Boolean indicating whether to include all longitudinal data, including data after the landmark age \code{x_L},
in the model development dataset. See Details section of \code{fit_LME_longitudinal} for more information.}

\item{cv_name}{Character string specifying the column name in \code{data_long} that indicates cross-validation fold}

\item{individual_id}{Character string specifying the column name in \code{data_long} which contains the individual identifiers}

\item{lme_control}{Object created using \code{nlme::lmeControl()}, which will be passed to the \code{control} argument of the \code{lme} function}
}
\value{
List containing elements:
\code{data_longitudinal}, \code{model_longitudinal}, \code{model_LME}, and \code{model_LME_standardise_time}.

\code{data_longitudinal} has one row for each individual in the risk set at \code{x_L} and
contains the value of the covariates at the landmark time \code{x_L} of the \code{predictors_LME} using the LOCF model and
\code{responses_LME} using the LME model.

\code{model_longitudinal} indicates that the LME approach is used.

\code{model_LME} contains the output from
the \code{lme} function from package \code{nlme}. For a model using cross-validation,
\code{model_LME} contains a list of outputs with each
element in the list corresponds to a different cross-validation fold.

\code{model_LME_standardise_time} contains a list of two objects \code{mean_response_time} and \code{sd_response_time} if the parameter \code{standardise_time=TRUE} is used. This
is the mean and standard deviation used to normalise times when fitting the LME model.
}
\description{
This function is a helper function for \code{fit_LME_landmark}.
}
\details{
For an individual \eqn{i}, the LME model can be written as

\deqn{Y_i = X_i \beta + Z_i U_i + \epsilon_i}

where
\itemize{
\item \eqn{Y_i} is the vector of responses at different time points for the individual
\item \eqn{X_i} is the matrix of predictors for the fixed effects at these time points
\item \eqn{\beta} is the vector of coefficients for the fixed effects
\item \eqn{Z_i} is the matrix of predictors for the random effects
\item \eqn{U_i} is the matrix of coefficients for the random effects
\item \eqn{\epsilon_i} is the error term, typically from N(0, \eqn{\sigma})
}

By using an LME model to fit repeat measures data, rather than a linear model, we can allow measurements from the same individuals to be
more similar than measurements from different individuals. This is done through the random intercept and/or
random slope.

Extending this model to the case where there are multiple random effects, denoted \eqn{k}, we have

\deqn{Y_{ik} = X_{ik} \beta_k + Z_{ik} U_{ik} + \epsilon_{ik}}

Typically the random effects are assumed to be from the multivariate normal (MVN) distribution \eqn{MVN(0,\Sigma_u)}
and we choose a certain covariance structure for \eqn{\Sigma_u}. The function \code{fit_LME_landmark} uses this distribution with
unstructured covariance for the random effects when fitting the LME model (i.e. no constraints are imposed on the values).

To fit the LME model the function \code{lme} from the package \code{nlme} is used.
The random intercept is always included in the LME model.
Additionally, the random slope can be included in the LME model using the parameter \code{random_slope_longitudinal=TRUE}.

It is important to distinguish between the validation set and the development set for fitting the LME model in this function.
The development dataset either includes all the repeat measurements (including those after the landmark age \code{x_L}), or only the repeat measurements
recorded up to and including the landmark age \code{x_L}. This is controlled using the parameter \code{include_data_after_x_L}.
The validation set only includes the repeat measurements recorded up until and including the landmark age \code{x_L},
i.e. it does not include future data in its predictions.

Using the fitted model, the values of the best linear unbiased predictions (BLUPs)
at the landmark age \code{x_L} are calculated. These BLUPs are the predictions of the values of the \code{responses_LME}
the landmark age \code{x_L}. The values of the predictors in this prediction are the LOCF values of the \code{predictors_LME}
at the landmark age \code{x_L}. In the function \code{fit_LME_landmark}, these predictions are used as covariates in the survival
model along with the LOCF values of  \code{predictors_LME}. Additionally, the estimated value of the random slope can
be included as predictors in the survival model using the parameter \code{random_slope_survival=TRUE}.

There is an important consideration about fitting the linear mixed effects model. As the variable \code{responses_LME_time}
gets further from 0, the random effects coefficients get closer to 0. This causes computational issues
as the elements in the covariance matrix of the random effects, \eqn{\Sigma_u}, are constrained to
be greater than 0. Using parameter \code{standard_time=TRUE} can prevent this issue by standardising the
time variables to ensure that the \code{responses_LME_time} values are not too close to 0.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_survival_model.R
\name{fit_survival_model}
\alias{fit_survival_model}
\title{Fit a survival sub-model as part of a landmarking analysis}
\usage{
fit_survival_model(
  data,
  individual_id,
  cv_name = NA,
  covariates,
  event_time,
  event_status,
  survival_submodel = c("standard_cox", "cause_specific", "fine_gray"),
  x_hor
)
}
\arguments{
\item{data}{Data frame containing covariates and time-to-event data, one row for each individual.}

\item{individual_id}{Character string specifying the column name in \code{data} which contains the individual identifiers}

\item{cv_name}{Character string specifying the column name in \code{data} that indicates cross-validation fold. If no cross-validation is needed, set this parameter to \code{NA}.}

\item{covariates}{Vector of character strings specifying the column names in \code{data_long}
which correspond to the covariates}

\item{event_time}{Character string specifying the column name in \code{data} which contains the event time}

\item{event_status}{Character string specifying the column name in \code{data} which contains the event status (where 0=censoring, 1=event of interest, if there are competing events these are labelled 2 or above). Events at time x_hor should be labelled censored.}

\item{survival_submodel}{Character string specifying which survival submodel to
use. Three options: the standard Cox model i.e. no competing risks (\code{"standard_cox"}),
the cause-specific regression model (\code{"cause_specific"}), or the Fine Gray
regression model (\code{"fine_gray"})}

\item{x_hor}{Numeric specifying the horizon time(s)}
}
\value{
List containing \code{data_survival} and \code{model_survival}

\code{data_survival} contains the predicted risk of event by the horizon time \code{x_hor}.

\code{model_survival} contains the outputs from the function used to fit the survival submodel, including the estimated parameters of the model.
For a model using cross-validation, \code{model_survival} contains a list of outputs with each
element in the list corresponding to a different cross-validation fold.
}
\description{
This function is a helper function for \code{fit_LOCF_landmark_model} and \code{fit_LME_landmark_model}.
}
\details{
This function fits the survival model from the landmark model framework. The individuals are censored at the time horizon \code{x_hor} and the survival model is fitted with
covariates specified in parameter \code{covariates}.

For the survival model, there are three choices of model:
\itemize{
\item the standard Cox model, this is a wrapper function for \code{coxph} from the package \code{survival}
\item the cause-specific model, this is a wrapper function for \code{CSC} from package \code{riskRegression}
\item the Fine Gray model, this is a wrapper function for \code{FGR} from package \code{riskRegression}
}

The latter two models estimate the probability of the event of interest in the presence of competing events.
}
\author{
Isobel Barrott \email{isobel.barrott@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_LOCF_landmark.R
\name{fit_LOCF_landmark}
\alias{fit_LOCF_landmark}
\title{Fit a landmark model using a last observation carried forward (LOCF) method for the longitudinal data}
\usage{
fit_LOCF_landmark(
  data_long,
  x_L,
  x_hor,
  covariates,
  covariates_time,
  k,
  cross_validation_df,
  individual_id,
  event_time,
  event_status,
  survival_submodel = c("standard_cox", "cause_specific", "fine_gray"),
  b
)
}
\arguments{
\item{data_long}{Data frame or list of data frames each corresponding to a landmark age \code{x_L} (each element of the list must be named the value of \code{x_L} it corresponds to).
Each data frame contains repeat measurements data and time-to-event data in long format.}

\item{x_L}{Numeric specifying the landmark time(s)}

\item{x_hor}{Numeric specifying the horizon time(s)}

\item{covariates}{Vector of character strings specifying the column names in \code{data_long}
which correspond to the covariates}

\item{covariates_time}{Vector of character strings specifying the column names in \code{data_long}
which contain the times at which \code{covariates} were recorded. This should either be length 1 or
the same length as \code{covariates}. In the latter case the order of elements must correspond to the
order of elements in \code{covariates}.}

\item{k}{Integer specifying the number of folds for cross-validation. An alternative to setting parameter \code{cross_validation_df} for performing cross-validation;
if both are missing no cross-validation is used.}

\item{cross_validation_df}{List of data frames containing the cross-validation fold each individual is assigned to. Each data frame in the list should be
named according to the landmark time \code{x_L} that they correspond. Each data frame should contain the columns \code{individual_id} and a column \code{cross_validation_number}
which contains the cross-validation fold of the individual. An alternative to setting parameter \code{k} for performing cross-validation;
if both are missing no cross-validation is used.}

\item{individual_id}{Character string specifying the column name in \code{data_long} which contains the individual identifiers}

\item{event_time}{Character string specifying the column name in \code{data_long}
which contains the event time}

\item{event_status}{Character string specifying the column name in \code{data_long}
which contains the event status (where 0=censoring, 1=event of interest, if there are competing events these are labelled
2 or above).}

\item{survival_submodel}{Character string specifying which survival submodel to
use. Three options: the standard Cox model i.e. no competing risks (\code{"standard_cox"}),
the cause-specific regression model (\code{"cause_specific"}), or the Fine Gray
regression model (\code{"fine_gray"})}

\item{b}{Integer specifying the number of bootstrap samples to take when calculating standard error of c-index and Brier score}
}
\value{
List containing containing information about the landmark model at each of the landmark times.
Each element of this list is named the corresponding landmark time, and is itself a list containing elements:
\code{data}, \code{model_longitudinal}, \code{model_survival}, and \code{prediction_error}.

\code{data}  has one row for each individual in the risk set at \code{x_L} and
contains the value of the \code{covariates} at the landmark time \code{x_L} using the LOCF approach. It also includes the predicted
probability that the event of interest has occurred by time \code{x_hor}, labelled as \code{"event_prediction"}.
There is one row for each individual.

\code{model_longitudinal} indicates that the longitudinal approach is LOCF.

\code{model_survival} contains the outputs from the function used to fit the survival submodel, including the estimated parameters of the model.
For a model using cross-validation, \code{model_survival} contains a list of outputs with each
element in the list corresponding to a different cross-validation fold. For more information on how the survival model is fitted
please see \code{?fit_survival_model} which is a function used within \code{fit_LOCF_landmark}.

\code{prediction_error} contains a list indicating the c-index and Brier score at time \code{x_hor} and their standard errors if parameter \code{b} is used.
For more information on how the prediction error is calculated
please see \code{?get_model_assessment} which is the function used to do this within \code{fit_LOCF_landmark}.
}
\description{
This function performs the two-stage landmarking analysis.
}
\details{
Firstly, this function selects the individuals in the risk set at the landmark time \code{x_L}.
Specifically, the individuals in the risk set are those that have entered the study before the landmark time \code{x_L}
(there is at least one observation for each of the \code{predictors_LME} and \code{random_effects} on or before \code{x_L}) and
exited the study after the landmark age (\code{event_time}
is greater than \code{x_L}).

Secondly, if the option to use cross validation
is selected (using either parameter \code{k} or \code{cross_validation_df}), then an extra column \code{cross_validation_number} is added with the
cross-validation folds. If parameter \code{k} is used, then the function \code{add_cv_number}
randomly assigns these folds. For more details on this function see \code{?add_cv_number}.
If the parameter \code{cross_validation_df} is used, then the folds specified in this data frame are added.
If cross-validation is not selected then the landmark model is
fit to the entire group of individuals in the risk set (this is both the training and test dataset).

Thirdly, the landmark model is then fit to each of the training datasets. There are two parts to fitting the landmark model: using the longitudinal data and using the survival data.
Using the longitudinal data is the first stage and is performed using \code{fit_LOCF_longitudinal}. See \code{?fit_LOCF_longitudinal} more for information about this function.
This function censors the
individuals at the time horizon \code{x_L} and fits the survival model. Using the survival data is the second stage and is performed using \code{fit_survival_model}. See \code{?fit_survival_model} more for information about this function.

Fourthly, the performance of the model is then assessed on the set of predictions
from the entire set of individuals in the risk set by calculating Brier score and C-index.
This is performed using \code{get_model_assessment}. See \code{?get_model_assessment} more for information about this function.
}
\examples{
library(Landmarking)
data(data_repeat_outcomes)
data_model_landmark_LOCF <-
   fit_LOCF_landmark(
     data_long = data_repeat_outcomes,
     x_L = c(60, 61),
     x_hor = c(65, 66),
     covariates =
       c("ethnicity", "smoking", "diabetes", "sbp_stnd", "tchdl_stnd"),
     covariates_time =
       c(rep("response_time_sbp_stnd", 4), "response_time_tchdl_stnd"),
     k = 10,
     individual_id = "id",
     event_time = "event_time",
     event_status = "event_status",
     survival_submodel = "cause_specific"
   )
}
\author{
Isobel Barrott \email{isobel.barrott@gmail.com}
}
